#! /usr/bin/env python
# -*- coding: utf-8 -*-

import os
import numpy as np
import scipy.optimize
import Manyo.SAS as ms
import Manyo as mm
import math
import utsusemi.SAS.ana.Reduction.HistBaseCommands as HBC
import utsusemi.SAS.ana.Reduction.CorrBaseCommands as CBC
"""
Base Functions for SAS
Trans.py
- TransmittanceCorrectionExp
- TransmittanceCorrectionPoly3
- FitExponentialWoBase
- FitExponential
- FitPolynomial3
- GetTransNormByMonCounts
- GetTransNormByMonCountsTS
- GetTransNormByMon
- GetTransNormByMonTS
- GetCalcTransmittance
- CreateCoefficientSet
- SetCoeffToTransmittance
"""
#########################################
def TransmittanceCorrectionExp(dat, params, calcError=False, useAngDep=False):
    """
    Transmittance correction by an exponential function, which is p0*exp(-p1*lambda)+p2.
    @param dat      (ElementContainerMatrix) or list of ElementContainerMatrix
    @param params   (PyObject) Function parameters and their errors
    @param calcError    (bool) True: Calculate an error w/ errors of parameters.
                            False: Calculate it w/o errors of parameters.
    @param useAngDep (bool) True: use angle dependence of calculation for transmittance correction
    """
    if type(dat)==list:
        dat_list=dat
    else:
        dat_list=[dat]
        params_list = [params]
    for a_dat,one_params in zip(dat_list,params_list):
        trans = ms.TransmittanceCorrection(a_dat)
        p = mm.ListToDoubleVector(one_params[0])
        e = mm.ListToDoubleVector(one_params[1])
        trans.SetErrorPropagation(calcError)
        trans.SetParameters(p, e, trans.TRANSMIT_TYPE_EXP, useAngDep)
        trans.Execute()
        hh = a_dat.PutHeaderPointer()
        hh.AddHeader("TransmittanceParameter", p)
        hh.AddHeader("TransmittanceParameterError", e)
        hh.AddHeader("TransmittanceFitFunc", "Exponential")
        del trans

#########################################
def TransmittanceCorrectionPoly3(dat, params, calcError=False, useAngDep=False):
    """
    Transmittance correction by a degree 3 polynomial function, which is p0+p1*lamb+p2*lamb^2+p3*lamb^3.
    @param dat      (ElementContainerMatrix)
    @param params   (PyObject) Function parameters and their errors
    @param calcError    (bool) True: Calculate an error w/ errors of parameters.
                            False: Calculate it w/o errors of parameters.
    @param useAngDep (bool) True: use angle dependence of calculation for transmittance correction
    """
    if type(dat)==list:
        dat_list = dat
        params_list = params
    else:
        dat_list=[dat]
        params_list = [params]
    for a_dat,one_params in zip(dat_list,params_list):
        print("one_params=",one_params)
        trans = ms.TransmittanceCorrection(a_dat)
        p = mm.ListToDoubleVector(one_params[0])
        e = mm.ListToDoubleVector(one_params[1])
        trans.SetErrorPropagation(calcError)
        trans.SetParameters(p, e, trans.TRANSMIT_TYPE_POLY3, useAngDep)
        trans.Execute()
        hh = a_dat.PutHeaderPointer()
        hh.AddHeader("TransmittanceParameter", p)
        hh.AddHeader("TransmittanceParameterError", e)
        hh.AddHeader("TransmittanceFitFunc", "Polynomial3")
        del trans

#########################################
def TransmittanceCorrPolyApproximation(dat, params, calcError=False):
    """
    Transmittance correction by a degree 3 polynomial function, which is exp(-S*thick) where S = p0+p1*lamb+p2*lamb^2+p3*lamb^3.
    @param dat      (ElementContainerMatrix)
    @param params   (PyObject) Function parameters and their errors
    @param calcError    (bool) True: Calculate an error w/ errors of parameters.
                            False: Calculate it w/o errors of parameters.
    """
    print("params = ", params)
    if type(dat)==list:
        dat_list = dat
        params_list = params
    else:
        dat_list=[dat]
        params_list = [params]
    for a_dat,one_params in zip(dat_list,params_list):
        print("one_params=",one_params)
        a_dat.PutHeaderPointer().Add("TransmittancePolyApproxThick", params[2])
        trans = ms.TransmittanceCorrection(a_dat)
        p = mm.ListToDoubleVector(one_params[0])
        e = mm.ListToDoubleVector(one_params[1])
        trans.SetErrorPropagation(calcError)
        trans.SetParameters(p, e, trans.TRANSMIT_TYPE_POLYAPPROX, False)
        trans.Execute()
        hh = a_dat.PutHeaderPointer()
        hh.AddHeader("TransmittanceParameter", p)
        hh.AddHeader("TransmittanceParameterError", e)
        hh.AddHeader("TransmittanceFitFunc", trans.TRANSMIT_TYPE_POLYAPPROX)
        del trans

#########################################
def PreprocessForFit(dat, iniVals):
    """
    Preprocessing for fitting.
    @params dat         (ElementContainer)
    @params iniVals     (str) initial parameters separated with space
    @retval (list, list, list, list) lists of lambda, intensity, error and initial parameters
    """
    x = dat.PutList("Lamb")
    y = dat.PutList("Intensity")
    e = dat.PutList("Error")
    ini_str_list = iniVals.split(" ")
    ini_list = []
    for s in ini_str_list:
        if s != "":
            ini_list.append( float(s) )
    return x, y, e, ini_list

#########################################
def PostprocessForFit(dat, p1, err, func_name):
    """
    Postprocessing for fitting.
    @params dat         (ElementContainer)
    @params p1          (list) - list of fitting result
    @params err         (list) - list of fitting error
    @params func_name   (str) - fitting function name
    """
    dat.AddToHeader("TransmittanceParameter", mm.ListToDoubleVector(p1))
    dat.AddToHeader("TransmittanceParameterError", mm.ListToDoubleVector(err))
    dat.AddToHeader("TransmittanceFitFunc", func_name)

#########################################
def FitExponentialWoBase(dat,iniVals="1.0 0.01",weighted=True):
    """
    An exponential fitting. The fitting function is p0*exp(-p1*lambda).
    @params dat         (ElementContainer) or list of ElementContainer
    @params iniVals     (str) initial parameters separated with space
    @params weighted    (bool) using weighted least squares
    @retval PyObj
    """
    if type(dat)==list:
        dat_list = dat
    else:
        dat_list = [dat]
    p1_list=[]
    err_list=[]
    for a_dat in dat_list:
        f = lambda p, x: p[0]*np.exp(-p[1]*x)
        x, y, e, ini_list = PreprocessForFit(a_dat, iniVals)
        p1, err = Fit(f, ini_list, x, y, e, weighted, omitZero=False)
        p1.append(0.0)
        err.append(0.0)
        PostprocessForFit(a_dat, p1, err, "Exponential")
        p1_list.append(p1)
        err_list.append(err)
    if len(p1_list)==1:
        return p1_list[0], err_list[0], "exp"
    else:
        return p1_list, err_list, "exp"

#########################################
def FitExponential(dat,iniVals="1.0 0.01 0.0001",weighted=True):
    """
    An exponential fitting. The fitting function is p0*exp(-p1*lambda)+p2.
    @params dat         (ElementContainer) or list of ElementContainer
    @params iniVals     (str) initial parameters separated with space
    @params weighted    (bool) using weighted least squares
    @retval PyObj
    """
    if type(dat)==list:
        dat_list=dat
    else:
        dat_list=[dat]
    p1_list=[]
    err_list=[]
    for a_dat in dat_list:
        f = lambda p, x: p[0]*np.exp(-p[1]*x) + p[2]
        x, y, e, ini_list = PreprocessForFit(a_dat, iniVals)
        p1, err = Fit(f, ini_list, x, y, e, weighted, omitZero=False)
        PostprocessForFit(a_dat, p1, err, "Exponential")
        p1_list.append(p1)
        err_list.append(err)
    if len(p1_list)==1:
        return p1_list[0], err_list[0], "exp"
    else:
        return p1_list, err_list, "exp"

#########################################
def FitPolyApproximation(dat, thick=1.0, iniVals="0.0001 0.0001 0.0001 0.0001", weighted=True):
    """
    An exponential fitting. The fitting function is
      exp(-(P1 + P2 * Lambda + P3 * Lambda^2 + P4 * Lambda^3) * T)
    @params dat         (ElementContainer) or list of ElementContainer
    @params thick       (float) thickness of sample [cm]
    @params iniVals     (str) initial parameters separated with space
    @params weighted    (bool) using weighted least squares
    @retval PyObj
    """
    if type(dat) == list:
        dat_list = dat
    else:
        dat_list = [dat]
    p1_list = []
    err_list = []
    for a_dat in dat_list:
        f = lambda p, x, t: np.exp(-(p[0] + (p[1] * x) + (p[2] * x * x) + (p[3] * x * x * x)) * t)
        x, y, e, ini_list = PreprocessForFit(a_dat, iniVals)
        #p1, err = Fit(f, ini_list, x, y, e, weighted, omitZero=False)
        p1, err = FitPolyApprox(f, ini_list, x, y, e, thick, weighted, omitZero=False)
        trans = ms.TransmittanceCorrection()
        a_dat.AddToHeader("TransmittancePolyApproxThick", thick)
        PostprocessForFit(a_dat, p1, err, trans.TRANSMIT_TYPE_POLYAPPROX)
        print("Fitting : exp(-(p0 + (p1 * lam) + (p2 * lam^2) + (p3 * lam^3)) * thick)")
        for i in range(4):
            print("p{} = {} +- {}".format(i, p1[i], err[i]))
        p1_list.append(p1)
        err_list.append(err)
    if len(p1_list) == 1:
        return p1_list[0], err_list[0], thick, trans.TRANSMIT_TYPE_POLYAPPROX
    else:
        return p1_list, err_list, thick, trans.TRANSMIT_TYPE_POLYAPPROX

#########################################
def FitPolynomial3(dat,iniVals="0.3 -0.05 0.004 -0.0001",weighted=True):
    """
    A degree 3 polynomial fitting. The fitting function is p0+p1*lamb+p2*lamb^2+p3*lamb^3.
    @params dat         (ElementContainer) or list of ElementContainer
    @params iniVals     (str) initial parameters separated with space
    @params weighted    (bool) using weighted least squares
    @retval PyObj
    """
    if type(dat)==list:
        dat_list=dat
    else:
        dat_list=[dat]
    ret_list = []
    for a_dat in dat_list:
        f = lambda p, x: p[0] + p[1]*x + p[2]*x**2 + p[3]*x**3
        x, y, e, ini_list = PreprocessForFit(a_dat, iniVals)
        p1, err = Fit(f, ini_list, x, y, e, weighted, omitZero=False)
        PostprocessForFit(a_dat, p1, err, "Polynomial3")
        ret_list.append( (p1,err,"poly3" ) )
    if len(ret_list)==1:
        return ret_list[0]
    return ret_list

#########################################
def GetTransNormByMonCounts(SmplRunNo=104000, DrctRunNo=104000, AxType="lambda", XRange="0.7 7.6 0.1", frameBoundary=0.0, CalibEffi=True, paramFiles="- -" ):
    """
    Monitor histogram of sample and direct followed by a transmittance calculation.
    @param SmplRunNo     (int)    Sample measurement run number
    @param DrctRunNo     (int)    Direct beam measurement run number
    @param AxType        (string)  Type must be "tof', "lambda" or "lamda2"
    @param XRange        (string) "startX endX widthX" separated with space (not comma)
    @param frameBoundary (float)
    @param CalibEffi     (bool)    Monitor-detector efficiency calibration
    @param paramFiles    (string) "<wiringInfo> <detectorInfo>[ <caseInfo>]"
    @retval list of Tr
    """
    import Com
    s = XRange.split()
    norm_range = "%s %s"%(s[0], s[1])

    smpl1s = HBC.GetMonHistSAS(SmplRunNo, 1, AxType, XRange, 0, "-1 -1", str(frameBoundary), CalibEffi, paramFiles=paramFiles)
    smpl2s = HBC.GetMonHistSAS(SmplRunNo, 2, AxType, XRange, 0, "-1 -1", str(frameBoundary), CalibEffi, paramFiles=paramFiles)
    if type(smpl1s)!=list:
        smpl1s = [smpl1s]
        smpl2s = [smpl2s]
    for smpl1, smpl2 in zip(smpl1s,smpl2s):
        CBC.NormByMonCounts(smpl2, smpl1, "Lamb", norm_range, 1.0)

    drct1 = HBC.GetMonHistSAS(DrctRunNo, 1, AxType, XRange, 0, "-1 -1", str(frameBoundary), CalibEffi)
    drct2 = HBC.GetMonHistSAS(DrctRunNo, 2, AxType, XRange, 0, "-1 -1", str(frameBoundary), CalibEffi)
    CBC.NormByMonCounts(drct2, drct1, "Lamb", norm_range, 1.0)

    ret = []
    for smpl2 in smpl2s:
        ret.append( Com.ExecFunctionCode("A/B", smpl2, drct2) )
    if len(ret)==1:
        return ret[0]
    return ret

#########################################
def GetTransNormByMonCountsTS(SmplRunNo=104000, DrctRunNo=104000, AxType="lambda", XRange="0.7 7.6 0.1", frameBoundary=0.0, CalibEffi=True, SmplTSlice="-1 -1", DrctTSlice="-1 -1", paramFiles="- -"):
    """
    Monitor histogram of sample and direct followed by a transmittance calculation.
    @param SmplRunNo     (int)    Sample measurement run number
    @param DrctRunNo     (int)    Direct beam measurement run number
    @param AxType        (string)  Type must be "tof', "lambda" or "lamda2"
    @param XRange        (string) "startX endX widthX" separated with space (not comma)
    @param frameBoundary (float)
    @param CalibEffi     (bool)    Monitor-detector efficiency calibration
    @param SmplTSlice    (string) Time Slice parameters for Sample run
    @param DrctTSlice    (string) Time Slice parameters for Directrun
    @param paramFiles    (string) "<wiringInfo> <detectorInfo>[ <caseInfo>]
    @retval Tr
    """
    import Com
    s = XRange.split()
    norm_range = "%s %s"%(s[0], s[1])

    smpl1s = HBC.GetMonHistSAS(SmplRunNo, 1, AxType, XRange, 0, SmplTSlice, str(frameBoundary), CalibEffi, paramFiles=paramFiles)
    smpl2s = HBC.GetMonHistSAS(SmplRunNo, 2, AxType, XRange, 0, SmplTSlice, str(frameBoundary), CalibEffi, paramFiles=paramFiles)
    if type(smpl1s)!=list:
        smpl1s = [smpl1s]
        smpl2s = [smpl2s]
    for smpl1,smpl2 in zip(smpl1s,smpl2s):
        CBC.NormByMonCounts(smpl2, smpl1, "Lamb", norm_range, 1.0)

    drct1 = HBC.GetMonHistSAS(DrctRunNo, 1, AxType, XRange, 0, DrctTSlice, str(frameBoundary), CalibEffi)
    drct2 = HBC.GetMonHistSAS(DrctRunNo, 2, AxType, XRange, 0, DrctTSlice, str(frameBoundary), CalibEffi)
    CBC.NormByMonCounts(drct2, drct1, "Lamb", norm_range, 1.0)

    ret = []
    for smpl2 in smpl2s:
        ret.append(Com.ExecFunctionCode("A/B", smpl2, drct2))
    if len(ret)==1:
        return ret[0]
    return ret

#########################################
def GetTransNormByMon(SmplRunNo=104000, DrctRunNo=104000, Xrange="0.7 7.6 0.1 dL", frameBoundary="0.0", CalibEffi=True, useMon="MON1", paramFiles="- -" ):
    """
    Monitor histogram of sample and direct followed by a transmittance calculation.
    @param SmplRunNo     (int)    Sample measurement run number
    @param DrctRunNo     (int)    Direct beam measurement run number
    @param Xrange        (string)  "startX endX widthX type" separated with space (not comma) : type=[dL, dL/L, tof]
    @param frameBoundary (float)
    @param CalibEffi     (bool)    Monitor-detector efficiency calibration
    @param useMon        (strng)  Upstream Monitor (MON1 or MON3)
    @param paramFiles    (string) "<wiringInfo> <detectorInfo>[ <caseInfo>]
    @retval Tr
    """
    import Com
    s = Xrange.split()
    norm_range = "%s %s"%(s[0], s[1])

    x_range = Xrange.split( " " )
    AxType=""
    if len(x_range)!=4:
        raise UserWarning("Xrange is invalid.")
    if x_range[3] == "dL":
        AxType = "lambda"
    elif x_range[3] == "dL/L":
        AxType = "lambda2"
    elif x_range[3] == "tof":
        AxType = "tof"
    else:
        raise UserWarning("Xrange is invalid.")
    XRange = "%s %s %s"%(x_range[0],x_range[1],x_range[2])

    monUpStream = 1
    if useMon=="MON1":
        monUpStream = 1
    elif useMon=="MON3":
        monUpStream = 3
    else:
        raise UserWarning("useMon is invalid.")

    smpl1s = HBC.GetMonHistSAS(SmplRunNo, monUpStream, AxType, XRange, 0, "-1 -1", frameBoundary, CalibEffi, paramFiles=paramFiles)
    smpl2s = HBC.GetMonHistSAS(SmplRunNo, 2, AxType, XRange, 0, "-1 -1", frameBoundary, CalibEffi, paramFiles=paramFiles)
    if type(smpl1s)!=list:
        smpl1s = [smpl1s]
        smpl2s = [smpl2s]
    for smpl1,smpl2 in zip(smpl1s,smpl2s):
        CBC.NormByMonCounts(smpl2, smpl1, "Lamb", norm_range, 1.0)

    drct1 = HBC.GetMonHistSAS(DrctRunNo, monUpStream, AxType, XRange, 0, "-1 -1", frameBoundary, CalibEffi)
    drct2 = HBC.GetMonHistSAS(DrctRunNo, 2, AxType, XRange, 0, "-1 -1", frameBoundary, CalibEffi)
    CBC.NormByMonCounts(drct2, drct1, "Lamb", norm_range, 1.0)

    ret = []
    for smpl2 in smpl2s:
        ret.append(Com.ExecFunctionCode("A/B", smpl2, drct2))
    if len(ret)==1:
        return ret[0]
    return ret

#########################################
def GetTransNormByMonTS(SmplRunNo=104000, DrctRunNo=104000, Xrange="0.7 7.6 0.1 dL", frameBoundary="0.0", CalibEffi=True, SmplTSlice="-1 -1", DrctTSlice="-1 -1", useMon="MON1", paramFiles="- -"):
    """
    Monitor histogram of sample and direct followed by a transmittance calculation.
    @param SmplRunNo     (int)    Sample measurement run number
    @param DrctRunNo     (int)    Direct beam measurement run number
    @param Xrange        (string)  "startX endX widthX type" separated with space (not comma) : type=[dL, dL/L, tof]
    @param frameBoundary (float)
    @param CalibEffi     (bool)    Monitor-detector efficiency calibration
    @param SmplTSlice    (string) Time Slice parameters for Sample run
    @param DrctTSlice    (string) Time Slice parameters for Directrun
    @param useMon        (strng)  Upstream Monitor (MON1 or MON3)
    @param paramFiles    (string) "<wiringInfo> <detectorInfo>[ <caseInfo>]
    @retval Tr
    """
    import Com
    s = Xrange.split()
    norm_range = "%s %s"%(s[0], s[1])

    x_range = Xrange.split( " " )
    AxType=""
    if len(x_range)!=4:
        raise UserWarning("Xrange is invalid.")
    if x_range[3] == "dL":
        AxType = "lambda"
    elif x_range[3] == "dL/L":
        AxType = "lambda2"
    elif x_range[3] == "tof":
        AxType = "tof"
    else:
        raise UserWarning("Xrange is invalid.")
    XRange = "%s %s %s"%(x_range[0],x_range[1],x_range[2])

    monUpStream = 1
    if useMon=="MON1":
        monUpStream = 1
    elif useMon=="MON3":
        monUpStream = 3
    else:
        raise UserWarning("useMon is invalid.")

    smpl1s = HBC.GetMonHistSAS(SmplRunNo, monUpStream, AxType, XRange, 0, SmplTSlice, frameBoundary, CalibEffi, paramFiles=paramFiles)
    smpl2s = HBC.GetMonHistSAS(SmplRunNo, 2, AxType, XRange, 0, SmplTSlice, frameBoundary, CalibEffi, paramFiles=paramFiles)
    if type(smpl1s)!=list:
        smpl1s = [smpl1s]
        smpl2s = [smpl2s]
    for smpl1,smpl2 in zip(smpl1s,smpl2s):
        CBC.NormByMonCounts(smpl2, smpl1, "Lamb", norm_range, 1.0)

    drct1 = HBC.GetMonHistSAS(DrctRunNo, monUpStream, AxType, XRange, 0, DrctTSlice, frameBoundary, CalibEffi)
    drct2 = HBC.GetMonHistSAS(DrctRunNo, 2, AxType, XRange, 0, DrctTSlice, frameBoundary, CalibEffi)
    CBC.NormByMonCounts(drct2, drct1, "Lamb", norm_range, 1.0)

    ret = []
    for smpl2 in smpl2s:
        ret.append(Com.ExecFunctionCode("A/B", smpl2, drct2))
    if len(ret)==1:
        return ret[0]
    return ret


#########################################
def GetCalcTransmittance(dat):
    """
    Get trnasmittance data using a fitting result
    @param dat      (ElementContainer) Observed transmittance which was used by fitting facades
    @retval Tr_calc
    """
    lambs = dat.PutList("Lamb")
    lambdas = dat.PutList("Lambda")
    hh = dat.PutHeader()
    p = list(hh.PutDoubleVector("TransmittanceParameter"))
    e = list(hh.PutDoubleVector("TransmittanceParameterError"))
    f = hh.PutString("TransmittanceFitFunc")
    trans = ms.TransmittanceCorrection()
    if f == "Exponential":
        ys = [p[0]*math.exp(-p[1]*x)+p[2] for x in lambs]
        es = [math.sqrt( (e[0]**2+(x**2)*(p[0]**2)*(e[1]**2)) * math.exp(-2*p[1]*x) + e[2]**2) for x in lambs]
    elif f == "Polynomial3":
        ys = [ p[0]+p[1]*x+p[2]*x**2+p[3]*x**3 for x in lambs]
        es = [math.sqrt( e[0]**2 + (x*e[1])**2 + (e[2]*x*x)**2 + (e[3]*x*x*x)**2 ) for x in lambs]
    elif f == trans.TRANSMIT_TYPE_POLYAPPROX:
        t = hh.PutDouble("TransmittancePolyApproxThick")
        ys = [ math.exp(-(p[0] + p[1] * x + p[2] * x * x + p[3] * x * x * x) * t) for x in lambs]
        es = []
        for x in lambs:
            st = (p[0] + p[1] * x + p[2] * x * x + p[3] * x * x * x) * t
            ex = -t * math.exp(-st)
            ee = (ex * ex) * ((e[0] * e[0]) + (e[1] * e[1] * x * x) + (e[2] * e[2] * x * x * x * x) + (e[3] * e[3] * x * x * x * x * x * x))
            es.append(math.sqrt(ee))
    else:
        raise UserWarning("Unknown fitting function name.")
    ec = mm.ElementContainer(hh)
    ec.Add("Lamb", lambs, "Ang.")
    ec.Add("Lambda", lambdas, "Ang.")
    ec.Add("Intensity", ys)
    ec.Add("Error", es)
    ec.SetKeys("Lambda", "Intensity", "Error")
    return ec

#########################################
def CreateCoefficientSet(coeff="1.0 0.01 0.0001", err="0.1 0.001 0.00001", funcType="exp"):
    """
    Create a coefficient set
    @param coeff    (str) coefficient separated with space
    @param err      (str) error separated with space
    @param funcType (str) function type. "exp" or "poly3"
    @retval PyObj
    """
    c = coeff.split(" ")
    e = err.split(" ")
    if len(c) != len(e):
        raise UserWarning("Num. of parameters dose not match.")
    if funcType not in ["exp", "poly3"]:
        raise UserWarning("Undefined function type.")
    try:
        c_num = list(map(float, c))
    except ValueError:
        raise UserWarning("param: %s"%sys.exc_info()[1])
    try:
        e_num = list(map(float, e))
    except ValueError:
        raise UserWarning("err: %s"%sys.exc_info()[1])
    return c_num, e_num, funcType

########################################
def SetCoeffToTransmittance(dat, params):
    """
    Set a coefficient set to transmittance data container.
    @param dat      (ElementContainer) Observed transmittance
    @param params   (PyObj) Coefficient set
    @retval None
    """
    trans = ms.TransmittanceCorrection()
    if params[2] == "exp":
        if (len(params[0]) != 3) or (len(params[1]) != 3):
            raise UserWarning("Number of parameters dose not match.")
        dat.AddToHeader("TransmittanceFitFunc", "Exponential")
    elif params[2] == "poly3":
        if (len(params[0]) != 4) or (len(params[1]) != 4):
            raise UserWarning("Number of parameters dose not match.")
        dat.AddToHeader("TransmittanceFitFunc", "Polynomial3")
    elif params[2] == trans.TRANSMIT_TYPE_POLYAPPROX:
        dat.AddToHeader("TransmittanceFitFunc", trans.TRANSMIT_TYPE_POLYAPPROX)
    else:
        raise UserWarning("Undefined function type.")
    p = mm.ListToDoubleVector(params[0])
    e = mm.ListToDoubleVector(params[1])
    dat.AddToHeader("TransmittanceParameter", p)
    dat.AddToHeader("TransmittanceParameterError", e)

########################################
def Fit(fitfunc, initialValues, lambValues, intValues, errValues, weighted=True, omitZero=False):
    """
    Fit transmmitance data by a fitting function.

    @param fitfunc (lambda function) a fitting function
    @param initialValues (list) - list of initial values
    @param lambValues (list) - list of lambda
    @param intValues (list) - list of intensity
    @param errValues (list) - list of error
    @params weighted    (bool) using weighted least squares
    @params omitZero (bool) omitting zero data points
    @retval (list, list) - lists of fitting result and error of each parameters
    """
    if weighted:
        errfunc = lambda p, x, y, sigma: (fitfunc(p, x) - y)/sigma
    else:
        errfunc = lambda p, x, y, sigma: fitfunc(p, x) - y

    np_lamb = np.array(lambValues)
    np_int = np.array(intValues)
    np_err = np.array(errValues)
    indicies = np.flatnonzero(np_int)
    if omitZero:
        result = scipy.optimize.leastsq(errfunc, initialValues[:], args=(np_lamb[indicies], np_int[indicies], np_err[indicies]), full_output=True)
    else:
        result = scipy.optimize.leastsq(errfunc, initialValues[:], args=(np_lamb, np_int, np_err), full_output=True)
    if result[-1] not in [1, 2, 3, 4]:
        raise UserWarning("The solution was not found.%s%s"%(os.linesep, result[-2]))
    p1=result[0]                    # solution
    errMat=result[1]                # variance-covariance matrix
    err=np.sqrt(np.diag(errMat))
    fvec=result[2]['fvec']          # residuals
    C = len(initialValues)
    N=len(lambValues) # N: number of data
    D=N-C                           # D: degree of freedom
    chisq=np.sum(fvec**2)/D            # reduced chi-square
    return p1.tolist(), err.tolist()

########################################
def FitPolyApprox(fitfunc, initialValues, lambValues, intValues, errValues, thick=1.0, weighted=True, omitZero=False):
    """
    Fit transmmitance data by a fitting function.

    @param fitfunc (lambda function) a fitting function
    @param initialValues (list) - list of initial values
    @param lambValues (list) - list of lambda
    @param intValues (list) - list of intensity
    @param errValues (list) - list of error
    @param thick (float) thickness of
    @params weighted (bool) using weighted least squares
    @params omitZero (bool) omitting zero data points
    @retval (list, list) - lists of fitting result and error of each parameters
    """
    if weighted:
        errfunc = lambda p, x, y, sigma, t: (fitfunc(p, x, t) - y)/sigma
    else:
        errfunc = lambda p, x, y, sigma, t: fitfunc(p, x, t) - y

    np_lamb = np.array(lambValues)
    np_int = np.array(intValues)
    np_err = np.array(errValues)
    indicies = np.flatnonzero(np_int)
    if omitZero:
        result = scipy.optimize.leastsq(errfunc, initialValues[:], args=(np_lamb[indicies], np_int[indicies], np_err[indicies], thick), full_output=True)
    else:
        result = scipy.optimize.leastsq(errfunc, initialValues[:], args=(np_lamb, np_int, np_err, thick), full_output=True)
    if result[-1] not in [1, 2, 3, 4]:
        raise UserWarning("The solution was not found.%s%s"%(os.linesep, result[-2]))
    p1 = result[0]                    # solution
    errMat = result[1]                # variance-covariance matrix
    err = np.sqrt(np.diag(errMat))
    fvec = result[2]['fvec']          # residuals
    C = len(initialValues)
    N = len(lambValues) # N: number of data
    D = N - C                         # D: degree of freedom
    chisq = np.sum(fvec**2) / D       # reduced chi-square
    return p1.tolist(), err.tolist()

#########################################
def GetTransRPMTNormByMon(SmplRunNo=104000, DrctRunNo=104000, PixelRange="150 150 152 152", AxType="lambda", XRange="0.7 7.6 0.1", frameBoundary=0.0, CalibEffi=True ):
    """
    Monitor histogram of sample and direct followed by a transmittance calculation.
    @param SmplRunNo     (int)    Sample measurement run number
    @param DrctRunNo     (int)    Direct beam measurement run number
    @param PixelRange    (string) "x0 y0 x1 y1"
    @param AxType        (string)  Type must be "tof', "lambda" or "lamda2"
    @param XRange        (string) "startX endX widthX" separated with space (not comma)
    @param frameBoundary (float)
    @param CalibEffi     (bool)    Monitor-detector efficiency calibration
    @retval Tr
    """
    import Com
    s = XRange.split()
    norm_range = "%s %s"%(s[0], s[1])

    smpl1 = HBC.GetMonHistSAS(SmplRunNo, 1, AxType, XRange, 0, "-1 -1", str(frameBoundary), CalibEffi)
    #smpl2 = HBC.GetMonHistSAS(SmplRunNo, 2, AxType, XRange, 0, "-1 -1", str(frameBoundary), CalibEffi)
    smpl2 =GetSumRPMTPixelArea(SmplRunNo, PixelRange, AxType, XRange, str(frameBoundary) )
    CBC.NormByMonCounts(smpl2, smpl1, "Lamb", norm_range, 1.0)

    drct1 = HBC.GetMonHistSAS(DrctRunNo, 1, AxType, XRange, 0, "-1 -1", str(frameBoundary), CalibEffi)
    #drct2 = HBC.GetMonHistSAS(DrctRunNo, 2, AxType, XRange, 0, "-1 -1", str(frameBoundary), CalibEffi)
    drct2 =GetSumRPMTPixelArea(DrctRunNo, PixelRange, AxType, XRange, str(frameBoundary) )
    CBC.NormByMonCounts(drct2, drct1, "Lamb", norm_range, 1.0)

    return Com.ExecFunctionCode("A/B", smpl2, drct2)


#########################################
def GetSumRPMTPixelArea( runNo, PixelRange, AxType, XRange, frameBoundary ):
    if AxType=="lambda":    #"0.7 7.6 0.1 dL"
        Xrange = XRange+" dL"
    elif AxType=="tof":
        Xrange = XRange+" tof"
    elif AxType=="lambda2":
        Xrange = XRange+" dL/L"
    elif AxType=="d":
        Xrange = XRange+" d"
    else:
        raise UserWarning( "Invalid AxType")

    rpmt2 = HBC.GetHistogramSAS(runNo, Xrange, "RPMT", TimeSlice="-1 -1", frameBoundary=frameBoundary, isTimeFoc=False, SolAngCor=True, paramFiles="- -" )
    return HBC.GetSumPixelArea( rpmt2, PixelRange )




# -*- coding: utf-8 -*-

# Resource object code
#
# Created: 金 6月 23 15:42:10 2017
#      by: The Resource Compiler for PySide (Qt v4.8.4)
#
# WARNING! All changes made in this file will be lost!

try:
    from PySide6 import QtCore
except:
    try:
        from PySide2 import QtCore
    except:
        from PySide import QtCore

qt_resource_data = b"\x00\x00\x0d8\x89PNG\x0d\x0a\x1a\x0a\x00\x00\x00\x0dIHDR\x00\x00\x000\x00\x00\x000\x08\x06\x00\x00\x00W\x02\xf9\x87\x00\x00\x00\x06bKGD\x00\xff\x00\xff\x00\xff\xa0\xbd\xa7\x93\x00\x00\x00\x09pHYs\x00\x00\x0d\xd7\x00\x00\x0d\xd7\x01B(\x9bx\x00\x00\x00\x07tIME\x07\xe0\x04\x01\x076\x01l\xf8\xc7\xe1\x00\x00\x0c\xc5IDATh\xde\xed\x99ypT\xc7\x9d\xc7\xbf\xfd\xae\xb9$\x8d\x84n\x0e!\xc9\x12h\x8d\x90\x1c\xc0 X@\x5c.\x10\x8e(\xe3J\xb0#\x88\xcde\xfb\x8f\xadu\xb2\xeb\xddr\xb6\xb2\x07\xb5e\xd7\x967\xa9T\x5c\xeb\xcd\xa6\xe28\x0e\x02\x04\x85B\xbc`\x0c\xce\xfa\xc0X\x10\x0e\x1d\x06t\xa3A#\xcb\xe8\x9a\x19I3#\xcd\xf1\x8e\xee\xde?\xe6@f\x85%\x81\x93\xfd\xc7]\xd5\xd5\xef\xf5\xbc\xe9\xf7\xfb\xf4\xaf\x7f\xdd\xbf\xdf\xef\x01\xdf\x94o\xca\x03\x15\xf2u\x0d\xf4\xee\xdc\xb9\x85HM\xddl\xb1\xd9\xd6\x11B\x161B2DE\x11@\x08a\x86AA\xa9\x87S\xdaf\xe8\xfa\xa7\x01\xa7\xf3\xecw\x86\x87;\xfe\xdf\x01\xde\xc9\xcdM\xb6\xa4\xa5\xfd\x9d\x92\x98\xb87\xa1\xa0 eVN\x8e`KOWL\x89\x89\x90m6\x80D\x86\xe7\x9c\xc3\x08\x04\xa0\xfa\xfd\x18w\xbb\xf5\x91\xde^:\xeep\xf8\xb4`\xb0\x9a\x8f\x8c\xfcd\xab\xc3\xe1\xfe\xb3\x02\xbc\xbbt\xa9\xd5\x9a\x98\xf8\x9a\x9c\x9c\xbc/\xfb\x91G\xc4\xd4\xa2\x22\xc5\xa4(\xa0\x03\x03\xa0n7\xb8\xdf\x0f\x16\x0c\x02\x8cE\x00\x00\x10\xab\x15Bb\x22\x84\x8c\x0c\x88\xd9\xd9\xd0U\x15\xc3]]z\x7fS\x13\xd3=\x9e\x83#>\xdfK;Z[\xc7\xff\xe4\x00\xe7*+\x9f\x108\x7f+{\xd9\xb2\xc4\xec\x92\x12\x19\x1e\x0ft\x87\x03\xcc\xe3\x018\x9f\xf4?\x9c\xf3H\x8d\xdc\x80\x03\x10\xd3\xd2 \x17\x16\x02\xe9\xe9p\xdd\xb8\xa1\xf7\xd7\xd7\x07Ccc\xcfU\x5c\xb8P\xfb'\x01\xe0\x07\x0e\x08\xe7.]z\xc3\x9a\x95\xb5\xaf\xf0\xb1\xc7\x14\x85RhMM\xe0~?@\xc8\x9d\x81\x08\x89\x83\xc4\x04\xfe?\x10\x8c\x81G\xfb\x84\x84\x04\x98\x96/\x87!\x8ap|\xf4\x916\xde\xdb{t$=}\xdf\x8e\xdaZ\xfa\xb5\x014,]*\x07\xb2\xb2\xdeM)((\xcf+/7\xd3\xb66\x18\xb7nE\xe5%q\xa1\xfd\xba\x8e\x80ah\xe3\x86\xa1\xe9\x86!SJ\x15\x10\xc2%Q\xd4dQ4\xac\xa2\xa8$\x0a\x82b\x91$\x80s0\xce#\x1a\xe1\x1cJa!\xa4\xe2b\xf4\xd6\xd5\x85\x87\xdb\xda\xea\xdd\x81\xc0\xe6\x1d\x97.\x85\x1e\x18\x80sN>\xd9\xb2\xe5\xbd\xcc\xd2\xd2\x0d\xf3\xca\xcaLZ]\x1d\xb8\xd7\x1b\x17\x9e\x03p\x85Bt8\x1c\xa6\x5c\xd7\xbb4M;e\x8c\x8f_\xf4|\xf6\xd9\xf5@(4\x04\x00)\xb3gg\xa7\x95\x94\x14\x8a\xa2\xb8\x85\x88\xe2\x0e\x22\x8aY\xa9\x8a\x22&\xcb\xb2Hb\x9a\xe1\x1c\xc2\xacY\xb0\xac[\x87\xbe\xabW5ws\xf3U^V\xb6~\xfd\x81\x03\xc6\x03\x01\x9c\xab\xa8\xf8\x8f\xd4\xa2\xa2\xe7\xf3\xd6\xacQ\xd4\xf3\xe7\xc1}>\x10B@D\x11~]\xe7\xb7GGUut\xf4\xbc\xa1\xaa/o\xb9z\xf5\xfat4ZWU\xb5\x94\x87B?\x13\x08Y\x9e.\xcb\xa6$Q$,\xba\xb4\x84\xe4dX6n\xc4\xe7\x17/j^\x87\xa3\xb6\xfc\xf4\xe9]_5\x96\xf0U?\xfea\xf5\xeao[33\x9f\xcf[\xbbVQ\xeb\xea\xc0\xbc\xde\xc8\x92\x89\x1a\xa1\xbd\xbc\x9c\x88f\xb3 X\xad\xd5\xd3\x15\x1e\x00\xd6\xd4\xd44\xae}\xe7\x9dry\xf6\xec'\x065\xcd5\xa4\xeb\x1a\x89\xda\x0f\xf5z\x11\xfa\xe4\x13\xcc_\xbdZ\xb1\xa4\xa6~\xb7n\xfb\xf6=\xf7\x05P\x9d\x99i3%%U?\xb4q\xa3\xa2\xb7\xb4\x80\x8d\x8c\xc4\x85\x07!`n7\xa4\xfe~\xfc\xc5\xd6\xad\x8a)%\xe57\x1fn\xdc\xf8\xf4Lw\xb4\x15o\xbc\xf1\x87\xcc\xd2\xd2%\xe3@\xe7mU\xd5b\xfd\x86\xc7\x03\xf5\xfau\xe4\xae[\xa7\x80\xb1\xd7\x9b\xf7\xef\xcf\x9c1@jf\xe6\xbf\xcf^\xb6\xcc\xa6\x18\x06\xf4\xae\xaex\xbf\xc19b\xb3e\xf4\xf4@\xea\xef\xc7\xa2\x8a\x0a\x93%=\xfd\xb7\x1f\xac]\xbbc\xa6\x10E\x07\x0e\xf4\x17\xec\xdf\xbfR\x03\xda\x5c\xba\xae\xc7\xfa\xb5\xceNH\x86\x81\x8cE\x8b\xcc\xa3CC\xff9#\x80\xe3II\xb3,\xd9\xd9\xfb2KK\x95pCC\xbc\xdf\xafi\xbc\xdb\xef\xd74J\xe3FL{z \xc6 \xb2\xb3\xab\xdf/+\x9b1D\xd6\xe6\xcd\x81YK\x96l\x0d06\xe27\x8c\xf8a\x12\xbcr\x05\xe9\xa5\xa52Q\x94\xca\xba\xa7\x9eZ0m\x80\xe4\xe2\xe2\x97\xe7\xaeX!p\xb7\x1b\xcc\xef\x07\x00P\xce1\x10\x0a\xa9cnw\xed-\xbf?\x1c\x83\x00\x00\xd6\xd3\x03y`\x00\x8b\x1f\x7f\xdc\x94\x90\x93S\xfd\xfe\xa3\x8f\xce\x18\xe2\xe1\x1f\xffx@HM}j\x98R\x8dE\xcf\x0e\xe6\xf3\x81\xb9\x5c\xc8\x5c\xb8P\xa4cc\xff0m\x009)\xe9\x99\xd4\x05\x0bd\xad\xb33\xde\xe7RUJU\xf5\xcc\xe6\xba\xba]\x81\x81\x81W\x1d^\xef\x97 hO\x0f\xe4\xc1A,\xae\xac4%\xe4\xe6V\x9f\xf9\xd6\xb7\xbe3S\x88Uo\xbey\x9e\x1b\xc6\xb9QJ\xe3[\xa7\xda\xde\x8e\x94\x05\x0bDH\xd2v~\xfc\xb88%\xc0\x07k\xd7.\xb6\xcd\x9e\x9d\xa2H\x12\xa8\xc7\x13\x99\x09\xce\xe1SU\xaas\xfeO\x00\xb0\xf9\xf2\xe5W\x02_|\xf1J\xd7\xe8hX5\xeel\xd3\xd4\xe9\x8chb\xdb6SRA\xc1\xe1wKJf\x0c\xc1\x15\xe5E\x9fap\x16\xbd\xd7].H\xb2\x0cKZ\x9a\xe9\x5cM\xcd\xba)\x018\xe7\x15\xc999D\x1f\x18\x88\xbb\x01c\xba\x0e\x1e\x0c6o:{\xb6-\xf6\xdc\xe6\x86\x86W\xc7\x9c\xce\x7f\xed\x1a\x1e\x0e\xab1Mp\x0e\xeatB\xea\xefGqe\xa5)y\xc1\x82\xc3\xa7\x8b\x8a\x9e\x9c\x09@\xf9\x89\x13]4\x10\xe8\x0a2\x16\xef\xd3\xfa\xfa\x90\x9c\x9d-\x13]\xaf\x98\x12\x80P\xba\xc6\x96\x9a\xaaP\xf7\x1d\x0f7\xc8\x98:>:z\xe2\xeeg+\xae]\xfb7\xbf\xc3q\xe0\xa6\xdb\x1dV\x0d#\xee\xdf\xc4 \x16WV\x9a\xec\x8b\x16\xd5\x9c,,\x9c\x11\x84\x11\x0a\x1d\x09r\xae\xc6\xac\x99\xba\xdd\xb0\xa5\xa7\x8bD\xd7WM\x090\xeep\xe4\xcb6[\xc4x\xa3\x1a\x08\x18\x06S\xfb\xfb\xcfL\xf6\xb2\xad\xad\xad\xaf\xf9::\xfe\xb9\xdd\xe5R\xc3\x86\x11w\x0b\x8c\xeen\x88\xfd\xfd(\xde\xb6\xcd4\xab\xa4\xa4\xe6d~\xfe\xb4!\x98\xae\xbf\x17\xe2\xf1U\x04\xea\xf3A\xb1\xdb\x01A\xc8\xbb\xfbY\xe9\xee\x8e\xb0\xdb\x9d\xca\x5c.\x04O\x9f\x067\x0c\x08\x82\x80!U\x95\xdcCC]\xf7z\xe1\xe3\x9d\x9d?9\xc5\x18\xef`\xec\x95\x85\x99\x99&\x93\x18\xb15\xa3\xbb\x1b\x22\xe7(\xae\xac45\xebz\xcd\x91\xee\xee'w\x02g\xa6\x02H\xa3\xb4m\xdc0L<:\x0e\x1d\x1f\x87d\xb5\x02\x84\xd8\xa7\xb6\x01\xc0\x22\x9aL`\x81\x00x(\x04=\x10@hh\x88\xbf\x00\x04\xbf\xea\xa5\xdb\xba\xba~\xea\xed\xe8\xf8\xdb\x8e\xc1A5\xe0\xf5\x82\x0e\x0e\xc2\xb8}\x1b\xea\x87\x1f\x82\xff\xf1\x8fXXZj2''\xff\xee\x080\xa5&\x9664\x18\x040x,\xbe\xd04\x08\xb2\x0c\x0a\x98\xa6\xb3\x8d\x92\x98/\xcf\xef\x04(\xd3r\xbb\xb79\x1c\xbf\x18\xb9v\xedG\x17N\x9c\x08z\x1b\x1b\xa1\xb7\xb5Aw8\x10:{\x16\xe4\xda5\xac\xda\xbf\xdfb\xb2\xdb\x0f\x1f\x01\xb6N1\x94B\x04!\x16\x8fN\xdca\xa6u\x0e\x84h8\x0c\x22\xcb\xf1\x07\x08!\xecm y:\x10\xdb\xfb\xfa~\xaeS\xfaR\xc3\xc0\xc0xP\xd7\xe36\xa165\x81\xb4\xb7c\xd5s\xcfYLv\xfb\xef\x8e\x02\x8f\xdfk\x0c\xd7\xa9S\xc9\x9cs!>k\x8a\x02\xa6\xeb\x80ahS\x02\x08\x80[\xf3\xf9@\x92\x93#\x1a\xe0\x1c\x12!\xaa\x09\x983]#\xdc\x05\xfc\x921\xf6\xf7\x8d\x83\x83\xc1\xc0\xdd\x10mmX\xb9\x7f\xbfE\xb2\xdbk\xef\x05\xd1u\xf2\xe4\xc3\xa2(\xc6\x85\x15l6\x18\xc1 \xb8\xae\xfb\xa7s\x0e\xb4\x84\xdcn\x88ii\xf1\xf85Q\x92$\x0e\xac\x9d\xc9V\xf8\xbd\x08\xc4K\x9f\xb9\x5c\xc1\x80\xaeG\xd4\xcf9\xd4\xc6F\x90\xd6V\xac\xdc\xb7\xef\x9e\x10\xea\xed\xdb\x7fi\x89\xba\xbe\x9cs\x88II\xd0\xbc^\xd0p\xf8\xd6\x94\x00\x14\xa8\xf3:\x9d\xaa\x9c\x93\x13\x8fiS\x15\xc5*\x12\xf2\xecLO\xd5\xef\x01\xbf4\x18\xfb\x9b\xeb\x1eOp\xe2r\x0a76\x02\xad\xad(\xdb\xbbwR\x88\xb1\xe6\xe6-\x82\xcbe6\x5c.\x18\x1e\x0f\x04\xab\x15\xbe\xeen\xc3\x08\x87\xcfO\x09\xc0\x80\x8f\x07;:\x0c\xa5\xa0 \x1e\xaf\xdae\x19\x0cx\xe4\x10\x90=S\x88\x9d\xc0\xaft\xc6~x}x8\x18\x9cpN\x84\x1b\x1a\x80\x96\x16\x94\xed\xd9\xf3%\x88\x83\xc0\x9c\xe0\xc0\xc0\x12\xa1\xbe\x1e\xc1+W\x10\xbc|\x19\x9cs\xdc\xaa\xad\x0d\x8d\xb6\xb5\x9d\x9c\x12\xe0\x19\xa0=80\xe0\x0b\x87\xc3\x90rr\xc0\xa3\xfe\x7f\x86\xc9D$Ax\xed~\xf2H\xbb\x807u\xce\x7fxcd\xe4K\x10\xa1\x86\x06\xb0\x96\x16,\xdf\xbd\xdb\x22\xd9\xed\xb5\x87\x81o+\xc0\xbf\xa4J\x92 D\x97\x8f<\x7f>\xb4P\x08\xe3\x83\x83\x86C\xd7\xafN\xcb\x1be\x9c\xbf\xd9\x7f\xf5\xaaf^\xbe<\xbe\x8c\xe6\x98\xcd\x0a\xe7\xfc\xbb5@\xd1\xfdB0\xce\x7f\xd0\xe2\xf5\x06C\x86\x11\xb7\x89p}=xs3\x96\xef\xddk1\xa7\xa4\x1c\x07!UY\x92d\x8am\xe1\xb6\x95+1T_\xafs\xe0\xd0\x01\x80M\x0b\x80\x02\xbf\xe8\xb9p\xc1\x10\xf3\xf3!\xa6\xa5E\x0c\x09\xc0<\x8bE&\x84\xbc\xf7\x16\x90x?\x10U\xc0\xaf\x19\xe7?h\xf5\xfb\xbf\xac\x89\xfaz\xf0\xb66,\xab\xaa\xb2\xe4X\xad\xb6\xd8\xec\x8b\x19\x19\x90\x1ez\x08\xce\xcb\x97\x0d\x00?\x9blLq\xd2\x9c'\x10x\x92\xd2,\x81\xf3\xd2\xf4\xf2r)\xdc\xd4\x04\x00H\x90$!\xcc\x98\x852\xb6\xea)\xe0Hm4w5\x93r\x02h\xda\x0e\x0c\x0d\xeb\xfa&\xbb$\xc9RTX\xad\xb7\x17\x18\x1f\x87<2\x02\xce\x18@\x08R\xaa\xaa\xd0w\xe9\x926\xe2t\x1e\xdf\x09\xfcvF11\x03\xfe\xb1\xfb\xfc\xf9\x80\xae(0\xafX\x11\x9f\xad<\xab\xd5l\x15\xc5\x15\x06!\x1f\x1c\x06\x92\xeeG\x13;\x81\xb7(\xe7/v\x06\x02\xc1\x10\xa5w\xce\x89\x9b7\xc1\xa2\xf7\xd6\x95+A\xcdft_\xbc\x18\x06\xf0\xd2\x8c\x83\xfa]\x80\x9f1\xf6L\xe3\xc1\x83!\xcb\x86\x0d\x90\xe6\xcc\x89\x1flE\x09\x09\x09i\xb2\xbcR \xe4F\x0dPr\xbf\x10\x06\xe7\xbf\xea\x0c\x854\x9d\xb1\xb8M\x80s(\xf3\xe6!a\xd3&\x5c?t(\xc8\x18{\xa1\x0a\xf0\xdcW^h'p&<8x\xb0\xf5\xe8\xd1p\xd2\xf7\xbf\x0f1##2[\x8ca\xbe\xc5b\xce\xb3X\xe6\x09\x84\x5c:F\xc8\x89c\xc0C\xd3\x15\xfe0\xf0p\x0dP#\x11\xf2\xfc|E\x89/\xa3\xd8\xbaOy\xf6Y\xb4\x1f;\x16\x0e\xb8\x5c\xa7v\x02\xc7\x1e,3\x07H\x83\xc0\x999\x8f>Z^\xf4\xc4\x13\x8a\xef\xd0!\x18}}\xf1\x9c(\xe7\x1c\x83\xaa\xaa\x0fj\x1a%\x80\x93q\xfe6\x05\xfe\x87\x03\x8eg\x80\xc09@\xba\x0d\xa4\x8b@.\x07\xd6\x8b\x84<\xc9\x81\x87\xd3%I\xca\x90$Y\x98 \xbc4g\x0eRv\xef\xc6\xadS\xa7\xb4\xde\x86\x86F\x0f\xb0\xfeE@}\xe0\xe4\xee\xdb\x80\xd9\x02\xfcw\xfa\xe2\xc5k\x8a\x9f~\xda\x1a\xfc\xf8c\x84\xeb\xeb\x01\xce\xef$\xbb\x00\x8cQ\x8aQ]\x0f\xfb\x0dCU\x19\xb3\xf1H\xbc\xc1DB\xc2\x12!\x9a]\x14\xcd\x89\x91\x8a\x899Q\x0e\xc0ZV\x06\xdb\x86\x0dh;z4\xe4joo\x12\x81-;\x80)\xbf\x17L;\xbd~\x1c\x10u\xe0u[F\xc6\xee%{\xf6\xd8dU\xc5\xd8\xc9\x93\xf1\xc0\x7f\x8a\x04q\xdc\x1d\x8e\xbb\xe9\xb1%\x93\x96\x86\xa4\xed\xdbAM&|V]\x1d\x08\xb9\x5c\xb5~\xe0\xf9\x17\x00\xfd\xebH\xaf\x0b\x00dD\x02\x09\x09\x80\xfc*P\x91'\x08?\xcf//\xb7\xe6n\xda$S\xa7\x13\xa1+W\xa0\xf5\xf4L\xfe\x81#\xf6M v\xcd98!PrsaY\xb1\x02R~>\xbe\xf8\xe8#\xfd\xd6\xa7\x9f\x86{\x19{\xf9G\xc0\xef\x01\x18\x88\x00\xa8\xd1\x96\xcd\x04@\x8c\x0a\xacLRe\x00J>\x90\xf2\xd7\xc0_e)JE\xde\xea\xd5\xe2\xdc\xb22I6\x9b\xa1\xdd\xbc\x09\xcd\xe9\x04u\xbbA\xbd^\xf0X\xb6B\x10 \xda\xed\x10\xd3\xd3\xa1\xe4\xe5AY\xb8\x10z8\x8c\xbe\xcb\x97i\xf7\xc5\x8b\xd4\xadi\x1f\xfe\x17\xf0z\x070\x0c@\x8b\x0a\xadMR\xd5\xc89{o\x80\xbb\x85\x9f\xec:\xdeW\x04d>\x0dl\xcf\x03\xd6'\xe7\xe4\xc8Y\x0b\x17\xca)\xf3\xe7\x8b\x96\xb44\xc8v{$\x8e\x05`\x04\x83\xd0}>\x84\xdcn\x8c~\xfe9\x1d\xbcyS\xf7\xf5\xf6\xd2\xcf\x81\xf3\xb5\xc0\xefo\x00\xfdw\x09y\xf7\xb5z/\x88\x99j \xae\x85\x89\xad\x08(\x8f\x01\x0b\x8a\x81\xe2\x5c\xa0$\x81\x90\xb9\x12\xe7I\xd1\xdf\xc1\x01\x9d\x02c\xe3\xc0\xed/\x80\x96\x16\xe0\xc6\xfb@\x07\xbd3\xdb\x13g\xfc^\xb3?-\x0d|\x95\x0d\xc8\xd1*Mh\xc5\x09m\xac\x0a\xd1q\xc9\x84\xf1\xf9\x84\xca\xa2B\xc4\xaa1\xa1\xd5'\xb4\xf7m\x03\xd35~2A\xe0X\x15\xa3\xfd\xc2$\x87$\x8bV\x1e\x15\x98M\xa8t\x02\xe07\xe5\x9b\xf2\xe7,\xff\x0b\x7f\xd7\xe2\x85]_\xb6\x1b\x00\x00\x00\x00IEND\xaeB`\x82\x00\x00\x07\x1e\x89PNG\x0d\x0a\x1a\x0a\x00\x00\x00\x0dIHDR\x00\x00\x000\x00\x00\x000\x08\x06\x00\x00\x00W\x02\xf9\x87\x00\x00\x00\x06bKGD\x00\xff\x00\xff\x00\xff\xa0\xbd\xa7\x93\x00\x00\x00\x09pHYs\x00\x00\x0d\xd7\x00\x00\x0d\xd7\x01B(\x9bx\x00\x00\x00\x07tIME\x07\xe0\x05\x11\x07!,A\xdda-\x00\x00\x06\xabIDATh\xde\xedYol\x14\xc7\x15\xff\xcd\xec\x9d\x1dc\x03\xb5\xc1`\xfe\x94F\x8e\x88\xa8\xb0eZ\x12h@1\x05\xa5T\xa5mZ\x0aQZ)\x1f*\xa5jTU\xfdTZ\xa9\xfdVU\xa8RLdT)R\xd5\x0f\x95\xf8\xd0*\xb42*)Iq\x05\x85`\xce\x90P'\xa9\x1a\xf2\xd7\xd8&\xf6\xd9\xf8\xec\xf3\x99\x8bo\xf7\xf6v\xe6\xbd~\xd8\xdd\xbb\xf5\xf9\xee\x8c\xcf\x17\xe1H\x9e\xd5\xd3\x9b\xdd\x9d\xd9\xfd\xfd\xde\xbcy\xf3f\x17X.\xcbe\xb9|\xa6\x8b(\xb7\xe3\x9ec\xa1$\x18++\x80`\xa6\xf7\x84Z\x05\x80\xcb\xe9\x1e*\xfb\xc5\x8c\xba\xe3\xcf\x9d\x02\x83@L`v5\xb1\x06\xb1\x9eu\xce\xd9\xeb\x94\xad+r\xc0`\xfc\xe1\xf4\xefj\x17\xc3?\xb48\xe3\x09\xf4\x0ctAS\x06\x8a\x1chR\xd0\xa4\xbc\xba\x93\xd5\x9a\x1chV\xd96\x8a\x1c\xd8\xca\xc4\xd1\xb6_,z\x00e\xa5<Q@\x14\xf4H!\x04 \x0ax*sE\xde,+\x01\xbd\xe8=!>\xf5I,\xefK\xe8`\x06c\x89\x8c\x80\x0b\x84\x8b\xe0dp\x01W!\xd0\xd2\x18\x81\xa0\x15}\x22\x05-\x9bG\x84H/\x1d\x17\xf2\xad<\xdb\xd2\xc5\x88\xb8\xed5\xab\xa54\x029\xb0s\xc9p\xf6\xf0\x8bf\x05\xe6%\xe2B\xe0\xd9$f\x11\xf2\x89xd\xfc\xfb\x8e\xb6\x97R\x14b\x80)`u\x9eC\x88\x91\xbb\xa7(\x03M\xce\xd2!\xe0\x83#&\x00\xec\xa5\x0a^T\x12\x0c\x7f\x0d\x13\x12`h\xa4u\x0aB\x16^\xd7*\x9eJ\xec=f\x8c0\x8bME\x17)\x09\x87\x99\xc3\xec\x85D\x06y\xe0\x18\x82\x01A9\xf0Ze\x90\xd2\xd3\x90\x06\x03,=\x17\x03\x84\x14\x10\x12z\xcf\xcfCT\x22\xdb\x1c\x8b\xbc\xa06.\x98\x00i\xf1\xa3\xc6\x86\xf5g~z\xe4\xb75+\xaaW\xban\x027\x19c&08L\xae\xf6\xd0\x12\xc0\x04!\x19\x82\x5c\xeb\x83\x09\x96}\x17\x96JB\x1a.\x1c\x7f~3\xbb\x04\x7f\xf2\xf4\xafC\xb9k\xb9\xc9n\xd9)tu\x9fJ\xcf\x98\x89g\xcbN\xa7\xf7\x1e\x0b?\xf7\xf9\xf5\xcd\x9d\xcf~\xebW5\xfd\xf1>$\xcc1h\xd2^f\xa9\xc0\x82@P`\xb8\xe7\x1a\xca\xd5\xac0\x93\x89c\xc6I\x80X\x17\x08\xa5\x1eX\x06\x98\x18\xcc\x00\x11\x83\x09h\xa8\xd9\x80=\x0f>\x85s\x97^2c\xd3c\xbf\xe9\xedP\xcf\x97\xc2h\x94\xba9\xdcK}\xabwLo\x18\x9f\x8a\xb6\xb6o\x7f2\x1c7Ga+\x13,4 \x18,hV\x1d\x820\xa3\xe2H\xa4\xa3\xb09\x05!\x19R\x8aY\x22\xb2\xda\xcd\x95|\x0d\x01\xac\xa8Z\x8d\xf6\x87\x9e\xc6\xa5\xd7_\xb5\xa2\xe3C\x7f\xed=\xa1\x7f9\xdf\x1c0\xe6k0|\x90\xcf\xd7L\x8c?\xae\xa17?\xf2\xd0\x81P,u\x1b\x9a3`A\x10\xbe\xb3K\x82\xc3\x16b\xd6\x10L\x95\x00\x0c\x864\x5c\xb0\xc2\xc8\x01\x17\x22XGN\x0b\x81\xb0Q\x8d}\xcd?\xc0\xcd\x0f\xder\xde\x1d\xf8\xef[vR\x7fo\xaco\xfe\x9cc^\x02\xb8\x0c\xde\xbc\x9b\xba>\x8e\xdd:\xb2\xe6s\xeb\x1a\xb6m\xfa\xb2\x9cH\xddv]G\xba\xd6\xb7i\x06\xe3\xa9~hd \x8d\x80\xc5\x0d\x91%\x92\x1d\x01!fY^\x08@\x0a\x89\xdd[\x0ec\x22\x16\xe3\x9e\xbe\x8bc\xec\xe8\xc7\xdfx\x11\xa9\x8a\x85\xd1k\x9d\xb0\x0cG\x1d<\xfb\xda\xa9\xc4\x9d\xa9\x11\xde\xba\xeeQ\x08\xe9Nj\x87-\xc4\xcc~\xb0\xd4\x90\x86\x80a\x08\x18!Wd\xc8\xaf\xcb\xbcs\xaf\x9dG\xb0u\xc3~h[\xe2b\xe4\x9fI\xe9\x84\xf7]\xeb\xc4T\xc5\xd7\x81\x9e\x93\x18cG\xec?\xdd\xfd\xc7\x14)\xe0\x0b\xf5\xad\x80 \xc4\xccA\x90\x07^\x1a.H_\xbb\xa0\xa5{-@\x22\xdb.$\xd0\xdc\xb0\x03\x8d\xd5\xcd8w\xe1\x8cEL\xdf\xbe\xda\x99\x1e\xf8\xd4\x16\xb2Hg\xe6f\xdaN\x1f=u\xf6\xf7\xd6\xea\xaa&\xacz`\x0d\x1c\x98\x01w\xc1\x5c\x12F\xce\xe2\xb3\xc0\x1b\x02\x8d\xb5[\xf0p\xfd^\x9c\xed>m\xd9\xb6\xf9\xb3H\x87\xeaY\xe8Bf,\xb4\xc3\xc85\xba\xd5\xb4KM\x0e\x8e~p`\xd7\xf6\xfd\xe1\xa43\x0e\x8b\x929\xf0\xd2%b\xf8\x9378\x0f<\xdf\x87\x00\xea\xc2\x0dxd\xfda\xbcr\xb1\xcb\xbc3\x19\xfdS\xa4C\x1d/g%6\xca\xe94\xdcK}\x0d_\xfad\xe3x<\xda\xf2\xd5\xb6\xc3\xe1\xc9\xf4 \x1cX\xb9Q\x08L`i\x04#\x91;q\xabd\x0dv7}\x1f\x91\x1bW\xec\x0f\xfb\xdf\xbb\x1aY\xa1\x9e\xc1\xe5\xf2\xb6hF\xb99\xc8\xf0A>_\x9fH\xb5\xa7\xac\xe4\xe6\xc7\xbex(t\xc7|\x1f$T`\x14\x02\x12\x08\x9dR\x84\xb0\xb3\xf1\xbb\x18\xb85\xa8#o\xf4\xdc\x0e\xcf8\x07\x86:Pv\x8aZ6\x01\x5c\x06o\xda\xa5\xbb&\xa7\xe3Gjkk\xeb[\xb6|\xc5\x18\xb3\xdew\x17/C\xcc\x11\x7f\x83\xbf}\xf5\x13\xb0\x12\xc4\xff8\x7f6\xc1\xec\xec\xee9\x89\xf8}\xcbF\xafu\xc2\x92\x19\xe7\xe0\x95\xde\x7fOO\xc4\xa6\xb8m\xed7\xe6\xe4)\x02\xb9\x05\xeb\xc1\xda\x9dx\xc0i\xc4\x99\x97\xbb,V\xfc\xf5\xc8\xf3\x18\xbd\xef[J?\xbc\xbe\xf2\xaf\x97M6\xab\xb1u\xd5^\xf8\xfb|\xf2r\x1cR\x8c5\xe1fl\xaa\xda\x81\x97\xce\xfc\xd9J;\xe9\x1f^}\xc1\xe9\xabD:mT\xe0\x19b\xf8\xba\x9e\xd8\xf8(\xde\xec\x1f\xfc\xe8\xc8c\xad_\x0b\xb3\xcc \xe9L\xb8\x99\x22\x03u\xa1\xb5hY\xf9M\xfc\xed\xef\x7f1\xe3S\x93'{;\xf4\x8b\x8b\xfd6[.\x01\xe9\xf5\x09yR\x05 \x0c\xa0j\xe4:\x8d\xack\xa5\xc4\xd0\xe8G\xfb\xf6\xb5=\x19\xfeDM\xc0t\xee\xa2\xda\xa8\xc3\xce\x86\xa3\xb8p\xa5;=00x!r\xc29\xe6\xf5\x09\x05\xc4\xf0D\xe4r\xd6\xca\x11\x90\xdeK\xc2E$K\x22z\x83\xde\xa9oI7\xc5\xe2\xa3\xdb\xda[\xbe\x13\x8e\xa7?F[\xc3!\xbcs\xf3\x7f\xea?}7\xfaov9\xcf\xa4\xa7\xb3\xfb\x90|\xf0\xf9\x22\xef\x85\x8cq\x8f\xd6\x0eZ=\x5c\x82PU\xf4m\x8e\xac\xda:\xb3+\xad\xec\xa6\xf6m\x87\x8d\xe8\xe80w_|uj\xe0\xbczj\xf2C\xd8%@KOD\x01]\x94\x88Qb\xa3\x13|p\xfe\xcb\xf2I\xe5\x08i\x84\xee\x8e\xd0e^\x1b;d\xabT\xdd\x95\xde\xd7\xec\xe8\x9b\xfa\xc7\xc3\xd7y4\xef\x99\xf9\x00E\x91\x8dV\xd9\x04\xf2-r\xcfu;\x09\xa5-\xbad\xad\xb8\xf3\xc4\xe4\x90:\xde\x7f\x8e^G66-J\x16\xb4\xa5\x94\x01\xeb\x97\xd2\x85\xa4\xd0\xb7v\x1f\x04\x15\x11=\x8f\xa6r\x7f1\xcd\x076\xdf\x1d\xc4<\x04\x82DJ\x11\x0a\x82\xaf\xd8?2Q\xc4\x87\x8b\xf9s\xbe\xff\x065\x15!\xa3\x17\x12F+\xf1\x89I\xe4g\x0e%\x9e[\xc8\xa7+\xf3\xa3`\xb9,\x97\xe5\xf2\xd9,\xff\x07m\x86\xc4\x97\xf3/\xd5\x15\x00\x00\x00\x00IEND\xaeB`\x82"
qt_resource_name = b"\x00\x0f\x0cB)\xc7\x00E\x00d\x00i\x00t\x00-\x00d\x00e\x00l\x00e\x00t\x00e\x00.\x00p\x00n\x00g\x00\x0b\x06}\x10'\x00G\x00o\x00-\x00d\x00o\x00w\x00n\x00.\x00p\x00n\x00g"
qt_resource_struct = b"\x00\x00\x00\x00\x00\x02\x00\x00\x00\x02\x00\x00\x00\x01\x00\x00\x00$\x00\x00\x00\x00\x00\x01\x00\x00\x0d<\x00\x00\x00\x00\x00\x00\x00\x00\x00\x01\x00\x00\x00\x00"
def qInitResources():
    QtCore.qRegisterResourceData(0x01, qt_resource_struct, qt_resource_name, qt_resource_data)

def qCleanupResources():
    QtCore.qUnregisterResourceData(0x01, qt_resource_struct, qt_resource_name, qt_resource_data)

qInitResources()



#!/usr/bin/python
# -*- coding: utf-8 -*-
"""
1次元プロッタの CUI ファサードクラス

History
[inamura 160322] - Add new CUI for logx
"""
import os
import sys
import thread
from time import sleep
from vis.WxCom import *
from vis.UtilPlot import *


#######################################
#  MPlot
####################################### 
class MPlot(object) :
    """
    プロッタのファサードクラス
    Python インタープリタ からプロッタを起動し
    制御するためのユーザインターフェイスを提供
    """
    # プロッタ通信クラスのインスタンスリスト
    plotList = []
    # plotList のロック用
    lck = thread.allocate_lock()

    #########################################
    def __init__(self,data=None, closeFlag = True):
        """
        コンストラクタ
        @param  data  プロッタに表示するデータ、省略可
        　　　　エレメントアレイまたはエレメントアレイのリスト
        @closeFlag   親プロセス終了時に、プロッタをクローズするか
        　　　　　　 否かのフラグ
        @retval 無し
        """ 
        self.plot = None
        # エレメントコンテナ変換クラスのインスタンス取得
        self.ce = ConvertEC()
        # テンポラリファイルクラスのインスタンス取得
        self.tp = TempFile()

        # プロッタの作成
        self.NewPlot(data, closeFlag)
        
    ######################################### 
    def NewPlot(self, data=None, closeFlag = True): 
        """
        プロッタを追加し、カレントプロッタとする
        @param  data  プロッタに表示するデータ、省略可
        　　　　エレメントアレイまたはエレメントアレイのリスト
        @param closeFlag  親プロセス終了時に、子プロセスを終了させるかどうかのフラグ
        @retval 無し
        """    
        # 引数リストを作成
        pno = len(self.plotList) +1
        strPno = "%d" % pno
        
        if closeFlag == True:
            cflag = "True"
        else:
            cflag = "False"
        
        arg = ["FastPlot.py", strPno, cflag]
        # プロセスID とプロッタ番号からポート番号を生成
        pxx = (os.getpid() % 500)*20
        portno = 10000+ pxx + pno

        # GUI通信クラスのインスタンス取得
        p = CommWx()
        
        # プロセス生成と通信接続に成功したなら
        if p.NewComm(portno, arg):
            # 旧カレントプロッタのアイコン化要求
            self._IconizeCurrent()
            
            self.lck.acquire()
            # 追加したプロッタをリストへ追加
            self.plotList.append(p)
            # ロックの解放
            self.lck.release()

            # 追加したプロッタをカレントプロッタとする。   
            self.plot = p
            
            # データが指定されていたら
            if not data == None:
                # データを追加
                self.AddData(data)
        else:
            print "\nError!!  Cannot create a new plotter!!."
    
    ######################################### 
    def ChangePlot(self, num=None): 
        """
        カレントプロッタを変更する
        @param  num  カレントにするプロッタ番号
        @retval 無し
        """
        if num == None:
            print "\nError!!  Input PlotNumber." 
        else:
            # 整数変換できるか
            try:
                pno=int(num)
            except:
                print '\nError!!  The argument must be integer.'
                return
            
            # プロッタ番号の範囲チェック
            if pno < 1 or num > len(self.plotList):
                # 引数エラー
                print "\nError!!  No such plotter."
                return
    
            # 指定されたプロッタが閉じられていなければ
            if self.plotList[pno-1].IsAliveComm():
                # 旧カレントプロッタのアイコン化要求    
                self._IconizeCurrent()
                # カレントプロッタを変更
                self.plot = self.plotList[pno-1]
                # 新カレントプロッタの復元要求    
                self.plot.PutCommand("doiconize 0\n")
            # 既に閉じている場合は
            else:
                # エラーメッセージを表示
                print "\nThe plotter(%d) has been already closed." % num
     
    ######################################### 
    def _IconizeCurrent(self): 
        """
        カレントプロッタをアイコン化する
        @param  無し
        @retval 無し
        """             
        # 先に(同一インスタンスの)カレントプロッタがあれば
        if self.plot!= None:
            # カレントプロッタが閉じられていなければ
            if self.plot.IsAliveComm():
                # 旧カレントプロッタのアイコン化要求
                self.plot.PutCommand("doiconize 1\n")
                

    ##[inamura 171220]-->
    #########################################
    def SetAutoScaleY(self, flag=False):
        """
        Set AutoScaseY flag on adding data to plotter
        @param flag  
        """
        if isinstance( flag, bool ):
            strCmd = "autoscaley %s\n" % (flag)
            self.plot.PutCommand(strCmd)
    ##<--[inamura 171220]
    #########################################
    def AddData(self, data=None, isAutoScale=False):
        """
        プロッタにデータを追加
        @param  data  プロッタに追加するデータ
        　　　　エレメントコンテナまたはエレメントコンテナのリスト
                (プロッタが読み込める形式のデータタプルも可)
        @retval 無し
        """ 
        if data == None:
            self._EmptyName('ElementContainer')
            return

        self.SetAutoScaleY(isAutoScale) ##[inamura 171220]
        
        lstDat = []
        # リストか
        if type(data)== type([]):
            # リスト中のエレメントコンテナ毎に変換
            for ec in data:
                ret = self.ce.ConvertEC(ec)
                if ret == None:
                    print "\nArgument is not acceptable."
                    return
                # Manyo データからPython データタイプへ変換
                lstDat.append(ret)
        # 単一データ
        else:
            # Manyo データからPython データタイプへ変換
            ret = self.ce.ConvertEC(data)
            if ret == None:
                print "\nArgument is not acceptable."
                return
            lstDat.append(ret)
                         
        # テンポラリファイル名称を取得
        temppath = self.tp.GetTempName()
        
        if temppath == None:
            # 異常終了
            print "\nError!! Cannot make a temporary file."   
            return

        # オブジェクトをシリアライズしてテンポラリファイルへ書き込む
        ret = self.tp.MakeTempFile(lstDat, temppath)
        # 正常終了か
        if ret == 0:
            # コマンド作成(改行コード付)
            strCmd = "add %s\n" % temppath
            # コマンドの送信要求をカレントプロッタへ送る
            self.plot.PutCommand(strCmd)
        else:
            # 異常終了
            print "\nError!! Cannot make a temporary file."       

    #########################################         
    def Remove(self, dnum=0):
        """
        プロッタからデータを削除
        @param  dnum  データ番号、0の場合全部
        @retval 無し
        """ 
        # データ番号　入力していない場合
        if dnum == None:
            self._EmptyName('Data Number')
        else:
        # 整数変換できるか
            try:
                dno = int(dnum)
            except:
                print '\nError!!  Argument must be integer.'
                return

        # コマンド作成
            strCmd = "remove %d\n" % dno
        # コマンドの送信要求をカレントプロッタへ送る
            ret = self.plot.PutCommand(strCmd)
        # 否定応答を受信したなら
            if ret == "NACK":
                print "Error!!  Data number."
        
    #########################################         
    def SetLog(self, flag=None):
        """
        ログモード設定
        @param  flag  ログモード True or False
        @retval 無し
        """ 
        if flag == True or flag == False:
            # コマンド作成
            strCmd = "log %d\n" % flag
            # コマンドの送信要求をカレントプロッタへ送る
            self.plot.PutCommand(strCmd)
        else:
            print "\nError!!  The argument must be True or False."
            
    ##[inamura 160322]-->
    #########################################         
    def SetLogX(self, flag=None):
        """
        ログモード設定
        @param  flag  ログモード True or False
        @retval 無し
        """ 
        if flag == True or flag == False:
            # コマンド作成
            strCmd = "logx %d\n" % flag
            # コマンドの送信要求をカレントプロッタへ送る
            self.plot.PutCommand(strCmd)
        else:
            print "\nError!!  The argument must be True or False."
            
    #########################################         
    def SetLogY(self, flag=None):
        """
        ログモード設定
        @param  flag  ログモード True or False
        @retval 無し
        """ 
        if flag == True or flag == False:
            # コマンド作成
            strCmd = "log %d\n" % flag
            # コマンドの送信要求をカレントプロッタへ送る
            self.plot.PutCommand(strCmd)
        else:
            print "\nError!!  The argument must be True or False."
            
    ##<--[inamura 160322]
    #########################################         
    def SetOverlay(self, flag=None):
        """
        オーバレイモード設定
        @param  flag  オーバレイモード True or False
        @retval 無し
        """ 
        if flag == True or flag == False:
            # コマンド作成
            strCmd = "overlay %d\n" % flag
            # コマンドの送信要求をカレントプロッタへ送る
            self.plot.PutCommand(strCmd)
        else:
            print "\nError!!  The argument must be True or False."    
        
    #########################################         
    def SetXScale(self, x0=None, x1=None ):
        """
        自動スケールモード設定
        @param  x0 X軸始値
        @param  x1 X軸終値
        @retval 無し
        """ 
        self._makeScaleCommand("xscale", x0, x1)
        
    #########################################         
    def SetYScale(self, y0=None, y1=None ):
        """
        自動スケールモード設定
        @param  y0 Y軸始値
        @param  y1 Y軸終値
        @retval 無し
        """ 
        self._makeScaleCommand("yscale", y0, y1)
        
    #########################################         
    def _makeScaleCommand(self, cmd, r0, r1):
        """
        自動スケールモード設定
        @param  cmd  送信コマンド
        @param  r0  引数1
        @param  r1  引数2
        @retval 無し
        """         
        # 引数無しか
        if r0==None and r1==None:
            # 自動モード(引数無し)コマンドの送信要求をカレントプロッタへ送る
            self.plot.PutCommand(cmd+'\n') 
        # マニュアル指定モード
        else:  
            # 数値か 
            try:
                rr0=float(r0)
                rr1=float(r1)
            except:
                print "\nError!!  The argument1 and 2 must be numeric."
                return
            # 始値 < 終値か
            if rr0 < rr1:
                #strCmd = "%s %.2f %.2f\n" % (cmd,rr0,rr1)
                strCmd = "%s %g %g\n" % (cmd,rr0,rr1)
       
            else:
                print "\nError!!  The argument2 must be larger then the argument1."
                return
            # コマンドの送信要求をカレントプロッタへ送る
            self.plot.PutCommand(strCmd)
        
    #########################################         
    def SetOffset(self, xoffset=None, yoffset=None):
        """
        オーバレイ時のオフセット指定
        @param  xoffset  X軸オフセット
        @param  yoffset  Y軸オフセット
        @retval 無し
        """ 
        # オフセットを入力していない場合
        if xoffset == None or yoffset == None:
            self._EmptyName('XOffset and YOffset')
        # 整数変換できるか
        else:
            try:
                xo = int(xoffset)
                yo = int(yoffset)
            except:
                print "\nError!!  Arguments must be integer."
                
            # Xオフセット範囲チェック
            if xo < 0 or xo > 200:
                print "\nError!!  Offset must be between 0 to 200." 
                return
            # Yオフセット範囲チェック
            if yo < 0 or yo > 200:
                print "\nError!!  Arguments must be between 0 to 200." 
                return
            # コマンド作成
            strCmd = "offset %d %d\n" % (xoffset,yoffset)
            # コマンドの送信要求をカレントプロッタへ送る
            self.plot.PutCommand(strCmd)

    #########################################         
    def SetPlotParam(self,dnum=None, **kargs):
        """
        プロットパラメータ指定
        @param  dnum     データNo  または "all"
        @param  **kargs   キーワード引数
           ls    　　線種　　　　　　　['-', ':','--', '-.','']
           hs        ヒストグラムスタイル　True or False 
        　 lw        線幅　　　　　　　1〜10
        　 lc        線色　　　　　　　['*', 'b', 'g', 'r', 'c', 'm', 'y', 'k']
        　 mk    　　マーカーの種類  　['', '.','o','s','v','d','+','x']
        　 ms        マーカーのサイズ　1〜10
        　 mc        マーカーの色　　　['*', 'b', 'g', 'r', 'c', 'm', 'y', 'k']
        　 eb    　　エラーバー指定    True or False
        　 es        エラーバーのキャップサイズ　1〜10
        　 sc        エラーバーの色　　['*', 'b', 'g', 'r', 'c', 'm', 'y', 'k']
        @retval 無し
        """ 
        # データ番号を入力しなかった場合
        if dnum == None:
            self._EmptyName('Data Number')
        else:
            # 整数変換できるか
            try:
                dno = int(dnum)
            except:
                print '\nError data number!! The argument 1 must be integer.'
                return
            # キーワード引き数の数を確認
            if len(kargs) == 0:
                print "\nError!!  It needs at least one keyword." 
                print 'Available keywords are "ls", hs", "lw", "lc", "mk", "ms", "mc", "eb", "es" and "ec".'
                return
                
            # コマンド作成
            strCmd = "plotparam %d" % dno
    
            # キーワードを取得
            for key in kargs.keys():
                kcmd = ""
                
                # 線種の指定か
                if key=="ls":
                    kcmd = self._CheckValue(TraceAtt.linestyle, key, kargs[key])
                # ヒストグラムスタイル指定か   
                elif key=="hs":
                    if kargs[key]==True:
                        kcmd = " hs=%d" % 1
                    elif kargs[key]==False:
                        kcmd = " hs=%d" % 0
                    else:
                        print '\nError value!!  The value for "hs" must be True or False.'    
                # 線幅の指定か    
                elif key=="lw":
                    kcmd = self._CheckNum(key, kargs[key])
                # 線色の指定か
                elif key=="lc":
                    kcmd = self._CheckValue(TraceAtt.color, key, kargs[key])
                
                # マーカの指定か    
                elif key=="mk":
                    kcmd = self._CheckValue(TraceAtt.marker, key, kargs[key])
                # マーカーサイズの指定か    
                elif key=="ms":
                    kcmd = self._CheckNum(key, kargs[key])
                # マーカー色の指定か
                elif key=="mc":
                    kcmd = self._CheckValue(TraceAtt.color, key, kargs[key])
                
                # エラーバー指定か   
                elif key=="eb":
                    if kargs[key]==True:
                        kcmd = " eb=%d" % 1
                    elif kargs[key]==False:
                        kcmd = " eb=%d" % 0
                    else:
                        print '\nError value!!  The value for "eb" must be True or False.'
                # キャップサイズの指定か    
                elif key=="es":
                    kcmd = self._CheckNum(key, kargs[key])
                # エラーバー色の指定か
                elif key=="ec":
                    kcmd = self._CheckValue(TraceAtt.color, key, kargs[key])
                              
                # キーワードエラー
                else:
                    print "\nError Keyword !!"
                    print 'Available keywords are "ls", "lw", "lc", "mk", "ms", "mc", "eb", "es" and "ec".'
                    return
                 
                # キーワード引数文字列が正常に作成できていたなかったなら
                if kcmd == "":
                    # 異常終了
                    return
    
                # コマンドにパラメータ文字列を連結    
                strCmd = strCmd + kcmd
                
            # 改行コードを付加
            strCmd = strCmd + '\n'
      
            # コマンドの送信要求をカレントプロッタへ送る
            ret = self.plot.PutCommand(strCmd)
            # 否定応答を受信したなら
            if ret == "NACK":
                print "\nError!!  Data number." 

    #########################################         
    def _CheckValue(self, lst, key, value):
        """
        指定されたキーの値をチェック
        @param  lst 　有効な値のリスト
        @param  key   キー文字列
        @param  value 値の文字列
        @retval キーワード引き数文字
        """                 
        # 指定された値はリスト中に存在するか
        if value in lst:
            # リスト中の値の順番を探す
            i = 0
            for item in lst:
                # リスト中にあるか
                if item == value:
                    # 文字列のキーワード引き数を作る
                    kcmd = " %s=%s" % (key, lst[i])
                    # 作成したキーワード引き数を返す
                    return kcmd
                i += 1
                
        # 不正な値だったなら
        else:
            # インストラクションメッセージを作成
            strMsg = "Avairable values for %s are " % key
            # 指定可能な値を列挙
            # 最後の2個の値を除き、カンマ付きで列挙
            for i in range(len(lst)-2):
                item = '"%s", ' % lst[i]
                strMsg = strMsg + item
            # 最後の2個を連結    
            i +=1
            j = i+1
            item = '"%s" and "%s"' % (lst[i], lst[j])
            strMsg = strMsg + item
            
            # メッセージを表示
            print "\nValue Error!!"
            print strMsg
            # ヌル文字列を返す
            return ""
        
    #########################################         
    def _CheckNum(self, key, value):
        """
        指定されたキーの値をチェック
        @param  key   キー文字列
        @param  value 値の文字列
        @retval 引き数文字
        """ 
        
        # 整数変換可か
        try:
            no = int(value)
        except:
            print '\nError!!  Value for "%s" must be integer.' % key
            return ""
        if no < 1 or  no > 10:
            print '\nError!! Value for "%s" must be between 1 to 10.' % key
            return ""
          
        # 数値のキーワード引き数を作る
        kcmd = " %s=%.d" % (key, no)
        return kcmd
    
    #########################################         
    def ChangeOrder(self, no1=None, no2=None):
        """
        指定されたキーの値をチェック
        @param  no1   変更前の順番
        @param  no2   変更後の順番
        @retval 無
        """ 
        # 引数を2つ入力していなかった場合
        if no1 == None or no2 == None:
            print "\nError!! Input OrderNumbers before change and after change."
        # 整数変換可か       
        else:
            try:
                n1 = int(no1)
                n2 = int(no2)
            except:
                print '\nError!! Arguments must be integer.'
                return ""
            if n1 < 1 or n2 < 1:
                print "\nError!!  Arguments must not be 0."
            # コマンドを作成し送信    
            strCmd = "shuffle %d %d\n" % (n1,n2)
            ret = self.plot.PutCommand(strCmd) 
            # 否定応答を受信したなら    
            if ret == "NACK":
                print "Data number error!!"     
                                        
    #########################################         
    def Close(self, plot=None):
        """
        カレントのプロッタ画面を閉じる
        @param  plot プロッタ番号
                    引数無しのときは、カレントプロッタ
                    "All" とすると、開いている全画面を閉じる
        @retval 無し
        """ 
        # カレントプロッタのクローズ要求か
        if plot == None:
            self.plot.PutCommand("close\n" )
            
        # 全画面クローズ要求か
        elif plot == 0:
            # 全プロッタに対し
            for pl in self.plotList:
                # 生存していれば
                if pl.IsAliveComm():
                    # クローズ要求を送信
                    pl.PutCommand("close\n" )
                    
        # 指定プロッタのクローズか
        else:
            # 数値か
            try:
                pnum=int(plot)
            except:
                print '\nError plotter number.'
                return
            
            # プロッタ番号の範囲チェック
            if pnum > 0 and pnum <= len(self.plotList):
                # 指定プロッタのインスタンス取得
                pl = self.plotList[pnum-1]
                # クローズ要求を送信
                pl.PutCommand("close\n" )
            else:
                # 引数エラー
                print "\nError!!  There is no such plotter."

    #########################################         
    def ChangePage(self, pageNo=None):
        """
        ページを変更するする
        @param  pageNo  ページ番号
        @retval 無し
        """ 
        if pageNo == None:
            self._EmptyName('Page No')
        else:
        # 整数変換できるか
            try:
                pno = int(pageNo)
            except:
                print '\nError!!  Argument must be integer.'
                return
            if pno > 0:
                strCmd = "page %d\n" % pno
                # ページ変更要求をカレントプロッタへ送る
                self.plot.PutCommand(strCmd)
            else:
                print '\nError!!  Argument must not be zero.' 
            
    #########################################         
    def SetTraceNum(self, traceNo=None):
        """
        ページを変更する
        @param  pageNo  トレース番号
        @retval 無し
        """
        # トレース番号　入力していない場合
        if traceNo == None:
            self._EmptyName('TraceNumber')
        else:
        # 整数変換できるか
            try:
                tno = int(traceNo)
            except:
                print '\nError!!  Argument must be integer.'
                return
            if tno > 0 and tno < 51:
                strCmd = "tracenum %d\n" % tno
            # ページ変更要求をカレントプロッタへ送る
                ret = self.plot.PutCommand(strCmd)
            # 否定応答を受信したなら

                if ret == "NACK":
                    print '\nError!! The number of trace must be less than 4 in indivisual mode.'   
            else:
                print '\nError!!  Argument must be between 1 to 50.'         

    #########################################         
    def SetXLabel(self, xlabel=None):
        """
        X軸のラベル設定
        @param  xlabel　横軸のラベル文字列
        @retval 無し
        """
        if xlabel == None or xlabel == "":
            self._EmptyName('xlabel')
        else:
            try:
                # 半角英数字を入力したか
                unicode(xlabel)
            except:
                self._AsciiName('xlabel')
            else:
            # コマンド作成
                strCmd = "xlabel %s\n" % xlabel
            # 座標ラベル変更要求をカレントプロッタへ送る
                self.plot.PutCommand(strCmd) 
        
    
    #########################################         
    def SetYLabel(self, ylabel=None):
        """
        X軸のラベル設定
        @param  ylabel  縦軸のラベル文字列
        @retval 無し
        """
        if ylabel == None or ylabel == "":
            self._EmptyName('ylabel')
        else:
            try:
                # 半角英数字を入力したか
                unicode(ylabel)
            except:
                self._AsciiName('ylabel')
            else:
            # コマンド作成
                strCmd = "ylabel %s\n" % ylabel
            # 座標ラベル変更要求をカレントプロッタへ送る
                self.plot.PutCommand(strCmd)     
        
    #########################################         
    def SetMainTitle(self, main=None):
        """
        メインタイトル設定
        @param  main　メインタイトル
        @retval 無し
        """
        if main == None or main == "":
            self._EmptyName('MainTitle')
        else:
            try:
                # 半角英数字を入力したか
                unicode(main)
            except:
                self._AsciiName('MainTitle')

            else:
                # コマンド作成
                strCmd = "maintitle %s\n" % main
                # タイトル変更要求をカレントプロッタへ送る
                self.plot.PutCommand(strCmd) 
        
    
    #########################################         
    def SetSubTitle(self, sub=None):
        """
        サブタイトル設定
        @param  sub　サブタイトル
        @retval 無し
        """
        if sub == None or sub == "":
            self._EmptyName('SubTitle')
        else:
            try:
                 # 半角英数字を入力したか
                unicode(sub)
            except:
                self._AsciiName('SubTitle')
            else:
        # コマンド作成
                strCmd = "subtitle %s\n" % sub
        # タイトル変更要求をカレントプロッタへ送る
                self.plot.PutCommand(strCmd)         
        
    #########################################         
    def Print(self):
        """
        カレントプロッタを印刷する
        @param  無し
        @retval 無し
        """ 
        # 印刷要求をカレントプロッタへ送る
        answer = self.plot.PutCommand("print\n" ) 
        # Windows の場合、タイムアウトによる NACK が返ってきたなら
        if answer == "NACK" and os.name == "nt" :
            print "\nPlease answer to the print dialog."
        
    #########################################         
    def Save(self, fname=None):
        """
        画像ファイルの保存
        @param  ファイル名
        @retval 無し
        """
        # 引数無し時のプロテクト
        if fname == None:
            self._EmptyName('filename')
            return
        try:
            unicode(fname)
    	except:
            # 全角文字が含まれている場合エラーとする
            self._AsciiName('Filename')
            return
        work = fname.split(os.sep)
        print work            
        # ファイル名取り出し
        filename = os.path.basename(fname)
        path, ext = os.path.splitext(filename)
        # 拡張子を除くファイル名の長さは1以上か
        if len(path) > 1:
            # サポートしている拡張子か
            if ext==".png" or ext==".ps" or ext==".eps" :
                strCmd = "save" + ' ' + fname + '\n'
                # 画像保存要求をカレントプロッタへ送る
                self.plot.PutCommand(strCmd) 
            else:
                print "\nError!!  Extention must be png, ps, or eps."
        else:
            print "\nError!! Invalid file name."

    #########################################
    def _AsciiName(self,name):
        """
        Ascii文字入力を促すメッセージ表示
        @param name 入力項目
        @retval 無し
        """
        print "\nCode error!! %s must be ASCII string." %name

    #########################################
    def _EmptyName(self,name):
        """
        引数を入力しなかった場合入力を促すメッセージ表示
        @param name 入力項目
        @retval 無し
        """
        print "\nMissing argument error!! Input %s." %name
        
    #########################################         
    def GetAttr(self, attr):
        """
        属性の取得
        @param  attr 属性を示す文字列
        @retval 属性(文字列)
        """ 
        # 属性のデータ型を取得
        try:
            typeno = TraceAtt.attrType[attr]
        except:
            print "\nError!! Attribute key. See following."
            strMsg = ""
            # キーの一覧を表示
            for att in TraceAtt.attrType.keys():
                strTmp = '%s, ' % att
                strMsg = strMsg + strTmp
            print strMsg    
        else:
            strComd = "getattr %s %d\n" % (attr, typeno)
            # 属性取得要求をカレントプロッタへ送る
            answer = self.plot.PutCommand(strComd )
            if answer == "NACK":
                return
            
            fr = file(answer,"rb")
            data = pickle.load(fr)
            # リストまたはディクショナリ
            if typeno == 4:
                try: 
                    # テンポラリファイルを読み込む
                    fr = file(answer,"rb")
                    # データを復元(デシリアライズ)
                    data = pickle.load(fr)
                except:
                    print "\nError!! Can't read temp file."
                else:
                    # テンポラリファイルをクローズ
                    fr.close()
                    # テンポラリファイルを削除
                    os.remove(answer)
                    # 復元したデータを表示
                    print "\n%s : " % attr
                    print data
            # 数値等    
            else:     
                print "\n%s : %s" % (attr, answer)            

                     
#####################################################################
if __name__ == '__main__':
    """
    以下はテスト用のスクリプト

    """
    #[inamura 121228: delete test script]

    # Added by VIC [
    MPlot(None, False)
    # ]



# -*- coding: utf-8 -*-

import os
import wx
import wx.xrc as xrc
import ana.Reduction.D4Mat_Manyo as D4Mat
import vis.M2Plot as M2P
import vis.D2Vis as D2Vis
import vis.UtilPlot as UtilPlot
import threading
import time



################################################
#                D4MatSlicer
#    GUI program to treat D4Matrix on Manyo-lib
#          Y.Inamura
#(2011.05.02) ver03 Revise treating axes titles (requred Manyo-lib >2250)
#(2011.04.28) ver02 Add Rebin func.
#(2011.04.11) ver01 firat version
################################################
__version__ = 'ver03(20110502)'

############################
## Load XMl resources file
############################
__XRCRESOURCEFILE = os.path.join( os.environ["UTSUSEMI_BASE_DIR"],'vis','D4MatSlicerP.xrc' )
#__XRCRESOURCEFILE = 'D4MatGUI.xrc'
__res = None

def get_resources():
    """ This function provides access to the XML resources in this module."""
    global __res
    if __res == None:
        __init_resources()
    return __res

def __init_resources():
    global __res
    __res = xrc.EmptyXmlResource()

    __res.Load(__XRCRESOURCEFILE)

###################################
## Make New Matrix Window
###################################
class MakeNewMatrixFrame(wx.Frame):
    def __init__(self,parent):
        """
        """
        self.parent = parent
        
        self.frame = get_resources().LoadFrame(parent.frame, 'NewD4MatrixFrame')
        self.frame.Bind( wx.EVT_BUTTON, self.OnLoadParams, id=xrc.XRCID("btLoadParams"))
        self.frame.Bind( wx.EVT_BUTTON, self.OnMakeNewMatrix, id=xrc.XRCID("btMakeNewMatrix"))
        self.frame.Bind( wx.EVT_BUTTON, self.OnClose, id=xrc.XRCID("btClose"))
        
        mpanel = xrc.XRCCTRL( self.frame, "NewD4MatrixPanel" )
        self.D4MatRangeCtrl = []
        self.D4MatTitle = []
        for i in range(12):
            self.D4MatRangeCtrl.append( xrc.XRCCTRL( mpanel, ( "txt%d" % (i+1) ) ) )
            
        for i in range(4):
            self.D4MatTitle.append( xrc.XRCCTRL( mpanel, ( "v%dtitle" % (i+1) ) ) )

        self.frame.Bind(wx.EVT_CLOSE, self.OnClose)
        self.CommentHead = "D4MatGUI::MakeNewMatrix >>> "
        # show window
        self.frame.Show()

    #################################
    def OnLoadParams(self,evt):
        """
        """
        print self.CommentHead+"OnLoadParams"
        # open dialog
        dlg = wx.FileDialog(self.frame, 'Open VisCont Parameter file ...', os.getcwd(), style=wx.OPEN, wildcard='XML(*.xml)|*.xml')

        # cancel?
        if dlg.ShowModal() != wx.ID_OK:
            return

        # get file path
        filepath = dlg.GetPath()
        dir = os.path.dirname(filepath)

        (ax_list,title_list) = self.parent.D4Mat.loadVisContParams(filepath)
        for i in range(4):
            ax = ax_list[i]
            self.D4MatRangeCtrl[ i*3 ].SetValue( str("%g" % ax[0]) )
            self.D4MatRangeCtrl[ i*3 + 1 ].SetValue( str("%g" % ax[1]) )
            self.D4MatRangeCtrl[ i*3 + 2 ].SetValue( str("%g" % ax[2]) )
            self.D4MatTitle[ i ].SetValue( title_list[i] )
        
        return

    #################################
    def OnMakeNewMatrix(self,evt):
        """
        """
        print self.CommentHead+"OnMakeNewMatrix"

        # open directory dialog
        dlg = wx.FileDialog( self.frame, 'Choose File to save D4Matrix parameters ...', defaultDir=os.getcwd(), defaultFile='D4Mat.xml', wildcard='xml File(*.xml)|*.xml',style=wx.SAVE )

        # cancel?
        if dlg.ShowModal() != wx.ID_OK:
            return

        # get path
        filepath = dlg.GetPath()
        dir = os.path.dirname(filepath)
        #print "filepath=",filepath
        #print "dir=",dir
        
        ax_list=[]
        title_list=[]
        for ax in range(4):
            ra = []
            for i in range(3):
                ra.append( float( self.D4MatRangeCtrl[ ax*3 + i ].GetValue() ) )
            ax_list.append(ra)
            title_list.append( self.D4MatTitle[ax].GetValue() )

        pre_label = self.parent.StatusTextCtrl.GetLabel()
        self.parent.StatusTextCtrl.SetLabel( "Making New D4Matrix files..." )
        self.parent.frame.Refresh()
        self.parent.frame.Update()
        self.frame.Refresh()
        self.frame.Update()
        
        self.parent.D4Mat.new( ax_list[0], ax_list[1], ax_list[2], ax_list[3], title_list, str(filepath) )
        ##self.parent.D4Mat.D4Mat._MakeNewParamXml( str(filepath)+".xml" ) ##[inamura 110411]
        for ax in ax_list:
            print self.CommentHead,ax
        
        self.parent.D4Mat.close()
        self.parent._OpenD4Matrix(str(filepath))
        self.parent.StatusTextCtrl.SetLabel( pre_label )

        dlg = wx.MessageDialog(self.frame, "Making New D4Matrix files completed.","",wx.OK)
        ret = dlg.ShowModal()
        
        return

    ##########################################################
    def OnClose(self, evt):
        """
        Close button event
        @param  evt event information
        @retval None
        """        
        self.parent.StatusTextCtrl.SetLabel( "Done." )
        self.frame.Destroy()
        

###################################
## Rebin Matrix Window
###################################
class RebinMatrixFrame(wx.Frame):
    def __init__(self,parent):
        """
        """
        self.parent = parent

        if not self.parent.D4Mat.isOpened():
            dlg = wx.MessageDialog(self.parent.frame, "You need to open D4Matrix at first!!","Warning",wx.OK)
            ret = dlg.ShowModal()
            return
        
        self.frame = get_resources().LoadFrame(parent.frame, 'RebinD4MatrixFrame')
        self.frame.Bind( wx.EVT_BUTTON, self.OnRebinMatrix, id=xrc.XRCID("btRebinMatrix"))
        self.frame.Bind( wx.EVT_BUTTON, self.OnClose, id=xrc.XRCID("btClose"))
        
        mpanel = xrc.XRCCTRL( self.frame, "RebinD4MatrixPanel" )
        self.D4MatRangeCtrl = []
        self.D4MatTitle = []
        self.RebinRatio = []
        for i in range(12):
            self.D4MatRangeCtrl.append( xrc.XRCCTRL( mpanel, ( "txt%d" % (i+1) ) ) )
            
        for i in range(4):
            self.RebinRatio.append( xrc.XRCCTRL( mpanel, ( "spn%dStepRatio" % (i+1) ) ) )
            self.D4MatTitle.append( xrc.XRCCTRL( mpanel, ( "v%dtitle" % (i+1) ) ) )

        self.frame.Bind(wx.EVT_CLOSE, self.OnClose)

        for i in range(4):
            self.D4MatRangeCtrl[ i*3 + 0 ].SetLabel( self.parent.D4MatRangeCtrl[ i*3 + 0 ].GetLabel() )
            self.D4MatRangeCtrl[ i*3 + 1 ].SetLabel( self.parent.D4MatRangeCtrl[ i*3 + 1 ].GetLabel() )
            self.D4MatRangeCtrl[ i*3 + 2 ].SetLabel( self.parent.D4MatRangeCtrl[ i*3 + 2 ].GetLabel() )
            self.D4MatTitle[ i ].SetValue( self.parent.D4MatTitle[ i ].GetValue() )

        self.CommentHead = "D4MatGUI::RebinMatrix >>> "
        
        # show window
        self.frame.Show()

    #################################
    def OnRebinMatrix(self,evt):
        """
        """
        print self.CommentHead+"OnRebinMatrix"

        # open directory dialog
        dlg = wx.FileDialog( self.frame, 'Choose File to save D4Matrix parameters ...', defaultDir=os.getcwd(), defaultFile='D4Mat.xml', wildcard='xml File(*.xml)|*.xml',style=wx.SAVE )

        # cancel?
        if dlg.ShowModal() != wx.ID_OK:
            return

        # get path
        filepath2 = dlg.GetPath()
        
        # get path2
        ( dir1, filename1 ) = self.parent.D4Mat.putOpenedPath()
        filepath1 = os.path.join( dir1,filename1 )
        
        ax_list=[]
        title_list=[]
        ratio_list=[]
        for ax in range(4):
            ra = []
            for i in range(3):
                ra.append( float( self.D4MatRangeCtrl[ ax*3 + i ].GetLabel() ) )
            ratio_list.append( float( self.RebinRatio[ax].GetValue() ) )
            print self.CommentHead+"Ratio="+str( ratio_list[-1] )
            ra[2] = ra[2]*ratio_list[-1]
            ax_list.append(ra)
            title_list.append( self.D4MatTitle[ax].GetValue() )

        for ax in ax_list:
            print self.CommentHead,ax
            
        pre_label = self.parent.StatusTextCtrl.GetLabel()
        self.parent.StatusTextCtrl.SetLabel( "Making New D4Matrix files..." )
        self.parent.frame.Refresh()
        self.parent.frame.Update()
        self.frame.Refresh()
        self.frame.Update()
        
        self.parent.D4Mat.new( ax_list[0], ax_list[1], ax_list[2], ax_list[3], title_list, str(filepath2) )
        self.parent.D4Mat.close()

        self.parent.StatusTextCtrl.SetLabel( "Rebinning D4Matrix ..." )
        self.parent.frame.Refresh()
        self.parent.frame.Update()
        self.frame.Refresh()
        self.frame.Update()
        
        self.parent.D4Mat.rebin( filepath1, filepath2 )        
        
        self.parent._OpenD4Matrix(str(filepath2))
        self.parent.StatusTextCtrl.SetLabel( pre_label )

        dlg = wx.MessageDialog(self.frame, "Rebinning D4Matrix completed.","",wx.OK)
        ret = dlg.ShowModal()
        
        return
        
        
    ##########################################################
    def OnClose(self, evt):
        """
        Close button event
        @param  evt event information
        @retval None
        """ 
        self.parent.StatusTextCtrl.SetLabel( "Done." )
        self.frame.Destroy()
        
###################################
## Main Window to treat D4Mat
###################################
class D4MatSlicer(wx.Frame):
    __version__ = 'v110411'
    def __init__(self, parent):
        """
        Constructor
        @param parent parent window id
        
        """
        
        ## Make window and set Event handler
        self.frame = get_resources().LoadFrame(parent,"D4MatGUIMainFrame")

        ## bind Button
        self.frame.Bind( wx.EVT_BUTTON, self.OnOpenD4Matrix, id=xrc.XRCID("btOpenD4Matrix"))
        self.frame.Bind( wx.EVT_BUTTON, self.OnCloseD4Matrix, id=xrc.XRCID("btCloseMatrix"))
        self.frame.Bind( wx.EVT_BUTTON, self.OnClearColumn, id=xrc.XRCID("btClearColumn"))
        self.frame.Bind( wx.EVT_BUTTON, self.OnSaveTitles, id=xrc.XRCID("btSaveTitles"))
        self.frame.Bind( wx.EVT_BUTTON, self.OnCloseD4Matrix, id=xrc.XRCID("btCloseMatrix"))
        self.frame.Bind( wx.EVT_BUTTON, self.OnAddSingleFile, id=xrc.XRCID("btAddSingleFile"))
        self.frame.Bind( wx.EVT_BUTTON, self.OnAddFilesInDir, id=xrc.XRCID("btAddFilesInDir"))
        self.frame.Bind( wx.EVT_BUTTON, self.OnRemoveFile, id=xrc.XRCID("btRemoveFile"))
        self.frame.Bind( wx.EVT_BUTTON, self.OnAutoFill, id=xrc.XRCID("btAutoFill"))
        self.frame.Bind( wx.EVT_BUTTON, self.OnSlice, id=xrc.XRCID("btSlice") )
        self.frame.Bind( wx.EVT_BUTTON, self.OnOut3DText, id=xrc.XRCID("btOut3DText") )

        self.D4MatrixFilePathCtrl = xrc.XRCCTRL( self.frame, "stPathToD4MatrixFiles" )
        self.StatusTextCtrl = xrc.XRCCTRL( self.frame, "txtStatus" )
        
        self.ButtonCtrlList = []
        self.ButtonCtrlList.append( xrc.XRCCTRL( self.frame, "btOpenD4Matrix" ) )
        self.ButtonCtrlList.append( xrc.XRCCTRL( self.frame, "btClearColumn" ) )
        self.ButtonCtrlList.append( xrc.XRCCTRL( self.frame, "btSaveTitles" ) )
        self.ButtonCtrlList.append( xrc.XRCCTRL( self.frame, "btCloseMatrix" ) )
        self.ButtonCtrlList.append( xrc.XRCCTRL( self.frame, "btAddSingleFile" ) )
        self.ButtonCtrlList.append( xrc.XRCCTRL( self.frame, "btAddFilesInDir" ) )
        self.ButtonCtrlList.append( xrc.XRCCTRL( self.frame, "btRemoveFile" ) )
        self.ButtonCtrlList.append( xrc.XRCCTRL( self.frame, "btAutoFill" ) )
        self.ButtonCtrlList.append( xrc.XRCCTRL( self.frame, "btSlice" ) )

        for btn_ctrl in self.ButtonCtrlList:
            btn_ctrl.Enable(False)
        for i in range(2):
            self.ButtonCtrlList[i].Enable()
        
        mpanel = xrc.XRCCTRL( self.frame, "D4MatSetPanel" )
        self.D4MatRangeCtrl = []
        self.D4MatTitle = []
        for i in range(12):
            self.D4MatRangeCtrl.append( xrc.XRCCTRL( mpanel, ( "txt%d" % (i+1) ) ) )
            
        for i in range(4):
            self.D4MatTitle.append( xrc.XRCCTRL( mpanel, ( "v%dtitle" % (i+1) ) ) )
            

        spanel = xrc.XRCCTRL( self.frame, "SlicePanel" )
        self.SliceAxis = []
        self.SliceRange = []
        self.SliceFoldCB = []

        for i in range(4):
            self.SliceAxis.append( xrc.XRCCTRL( spanel, ( "v%dcb" % (i+1) ) ) )
            #self.SliceAxis[-1].Bind(wx.EVT_COMBOBOX, self.OnSliceAxis, self.SliceAxis[-1])
            self.SliceFoldCB.append( xrc.XRCCTRL( spanel, ( "cks%d" % (i+1) ) ) )
        for i in range(12):
            self.SliceRange.append( xrc.XRCCTRL( spanel, ( "txt%d" % (i+1) ) ) )

        menubar = get_resources().LoadMenuBar("MainMenuBar")
        self.frame.SetMenuBar( menubar )
        self.frame.Bind(wx.EVT_MENU, self.OnOpenMakeNewFrame, id=xrc.XRCID('menu_new_matrix'))
        self.frame.Bind(wx.EVT_MENU, self.OnOpenD4Matrix, id=xrc.XRCID('menu_open_matrix'))
        self.frame.Bind(wx.EVT_MENU, self.OnOpenRebinFrame, id=xrc.XRCID('menu_rebin_matrix'))
        #self.frame.Bind(wx.EVT_MENU, self.OnSaveDataAsText, id=xrc.XRCID('menu_saveas_text'))
        self.frame.Bind(wx.EVT_MENU, self.OnClose, id=xrc.XRCID('menu_exit'))
        
        self.FileListBox = xrc.XRCCTRL( self.frame, "lbFileList" )
        
        ## initialize
        for i in range(4):
            """
            self.D4MatRangeCtrl[ i*3 ].SetValue("-2.0")
            self.D4MatRangeCtrl[ i*3 + 1 ].SetValue("2.0")
            self.D4MatRangeCtrl[ i*3 + 2 ].SetValue("0.05")
            """
            self.D4MatRangeCtrl[ i*3 ].SetLabel("-2.0")
            self.D4MatRangeCtrl[ i*3 + 1 ].SetLabel("2.0")
            self.D4MatRangeCtrl[ i*3 + 2 ].SetLabel("0.05")
            if i != 3:
                self.D4MatTitle[ i ].SetValue( ("Q%d" % (i+1) ) )
            else:
                self.D4MatTitle[ i ].SetValue( "hw" )
                
                            
        self.frame.Show()

        self.D4Mat = D4Mat.D4Mat()
        self.ParamFilePath = ""
        self.CommentHead = "D4MatGUI >>> "
        self.M2Plot = None

    #################################        
    def OnOpenD4Matrix(self,evt):
        """
        """
        print self.CommentHead,"OnOpenD4Matrix"
        # open dialog
        dlg = wx.FileDialog(self.frame, 'Open D4Matrix param file ...', os.getcwd(), style=wx.OPEN, wildcard='XML(*.xml)|*.xml')

        # cancel?
        if dlg.ShowModal() != wx.ID_OK:
            return

        # get file path
        self.ParamFilePath = str( dlg.GetPath() )
        #dir = os.path.dirname(filepath)
        self._OpenD4Matrix()

    ################################
    def _OpenD4Matrix(self,filepath=""):
        if filepath=="":
            if self.ParamFilePath == "":
                print self.CommentHead + " Not set param file path.."
                return
        else:
            if os.path.exists( filepath ):
                self.ParamFilePath = filepath
            else:
                print self.CommentHead + " Not set param file path.."
                return
        
        ret = self.D4Mat.open(self.ParamFilePath)
        if (ret!=-1):
            self.D4MatrixFilePathCtrl.SetLabel(self.ParamFilePath)
        else:
            return

        for i in range(4):
            vv = self.D4Mat.D4Mat.PutAxRange(i)
            """
            self.D4MatRangeCtrl[ i*3 ].SetValue(str(vv[0]))
            self.D4MatRangeCtrl[ i*3 + 1 ].SetValue(str(vv[1]))
            self.D4MatRangeCtrl[ i*3 + 2 ].SetValue(str(vv[2]))
            """
            self.D4MatRangeCtrl[ i*3 ].SetLabel(str(vv[0]))
            self.D4MatRangeCtrl[ i*3 + 1 ].SetLabel(str(vv[1]))
            self.D4MatRangeCtrl[ i*3 + 2 ].SetLabel(str(vv[2]))
            sv = self.D4Mat.D4Mat.PutAxTitle(i)
            self.D4MatTitle[ i ].SetValue( sv )

        self.FileListBox.Clear()
        FileCompsVec = self.D4Mat.D4Mat.PutFileComponents()
        for i in range(FileCompsVec.size()):
            self.FileListBox.Append( FileCompsVec[i] )
        
        for btn_ctrl in self.ButtonCtrlList:
            btn_ctrl.Enable(True)
        

    #################################
    def OnCloseD4Matrix(self,evt):
        """
        """
        print self.CommentHead+"OnCloseMatrix"
        
        self._CloseD4Matrix()

        
    #################################
    def OnClearColumn(self,evt):
        """
        """
        print self.CommentHead+"OnClearColumn"
        
        for i in range(4):
            vv = self.D4Mat.D4Mat.PutAxRange(i)
            """
            self.D4MatRangeCtrl[ i*3 ].SetValue(str(vv[0]))
            self.D4MatRangeCtrl[ i*3 + 1 ].SetValue(str(vv[1]))
            self.D4MatRangeCtrl[ i*3 + 2 ].SetValue(str(vv[2]))
            """
            self.D4MatRangeCtrl[ i*3 ].SetLabel(str(vv[0]))
            self.D4MatRangeCtrl[ i*3 + 1 ].SetLabel(str(vv[1]))
            self.D4MatRangeCtrl[ i*3 + 2 ].SetLabel(str(vv[2]))
            sv = self.D4Mat.D4Mat.PutAxTitle(i)
            self.D4MatTitle[ i ].SetValue( sv )

    #################################
    def OnSaveTitles(self,evt):
        """
        """
        titles = []
        for i in range(4):
            tt = self.D4MatTitle[ i ].GetValue()
            titles.append( tt )
        self.D4Mat.setTitles( titles, True )

    #################################
    def OnAddSingleFile(self,evt):
        """
        """
        print self.CommentHead+"OnAddSingleFile"

        # open dialog
        dlg = wx.FileDialog(self.frame, 'Choose VisualCont2 output file ...', os.getcwd(),
                            style=wx.OPEN, wildcard='Binary(*.vbin)|*.vbin|TEXT(*.txt)|*.txt')

        # cancel?
        if dlg.ShowModal() != wx.ID_OK:
            return
        
        filepath = str( dlg.GetPath() )
        self.StatusTextCtrl.SetLabel( "Loading file %s" % (filepath ) )
        for btn_ctrl in self.ButtonCtrlList:
            btn_ctrl.Enable(False)

        self.frame.Refresh()
        self.frame.Update()
        
        self.D4Mat.addFromFile( filepath )
        self.StatusTextCtrl.SetLabel( "Done." )
        
        self.FileListBox.Append( filepath )

        for btn_ctrl in self.ButtonCtrlList:
            btn_ctrl.Enable(True)
        

    #################################
    def OnAddFilesInDir(self,evt):
        """
        """
        print self.CommentHead+"OnAddFilesInDir"
        
        #dlg = wx.DirDialog(self.frame, "Select Directory including VisualCont2 output files ...",os.getcwd(), style=wx.OPEN)
        dlg = wx.DirDialog(self.frame, "Select Directory including VisualCont2 output files ...",os.getcwd())
        if dlg.ShowModal() !=wx.ID_OK:
            return
        
        print self.CommentHead,"DIR=",dlg.GetPath()

        #check selected dir
        filedir = str(dlg.GetPath())
        if not os.path.exists( filedir ):
            print self.CommnetHead + "Cannot find such directory, "+filedir
            raise
        
        #show progress bar
        pdg = wx.ProgressDialog("Loading","Loading VisualCont2 output text files :",len( os.listdir(filedir) ))
        #start sub-thread
        #th = threading.Thread( target=self.D4Mat.addFromTextDir,args=(str( dlg.GetPath() ), ) )
        th = threading.Thread( target=self.D4Mat.addFromDirectry,args=(str( dlg.GetPath() ), ) )
        #th.setDaemon(True)
        th.start()
        # progress
        pdg.Update(self.D4Mat.progress)
        while self.D4Mat.progress <= self.D4Mat.progress_max:
            wx.Sleep(0.5)
            pdg.Update(self.D4Mat.progress)
        pdg.Destroy()
        
        #self.D4Mat.addFromTextDir( str( dlg.GetPath() ) )

        """
        self.FileListBox.Clear()
        
        FileCompsVec = self.D4Mat.D4Mat.PutFileComponents()
        for i in range(FileCompsVec.size()):
            self.FileListBox.Append( FileCompsVec[i] )
        """
        for file in self.D4Mat.AddedFileList:
            self.FileListBox.Append( file )
        
    #################################
    def OnRemoveFile(self,evt):
        """
        """
        print self.CommentHead+"OnRemoveFile"
        selected_list = self.FileListBox.GetSelections()
        
        if len(selected_list)==0:
            return
        else:
            print self.CommentHead,"selected ",selected_list
            print self.CommentHead,"len=",len(selected_list)
            last_index = len(selected_list)-1
            for i in range(last_index+1):
                ind = selected_list[ last_index - i ]
                filepath = self.FileListBox.GetString(ind)
                print self.CommentHead,"Removing File = ",filepath
                ret = self.D4Mat.subtractText( filepath )
                if ret==0:
                    self.FileListBox.Delete(ind)
                

    #################################
    def OnAutoFill(self,evt):
        """
        """
        print self.CommentHead+"OnAutoFill"
        
        #for i in range(12):
        #    self.SliceRange[i].SetValue( self.D4MatRangeCtrl[i].GetValue() )
        for i in range(4):
            vv = self.D4Mat.D4Mat.PutAxRange(i)
            for j in range(3):
                self.SliceRange[ 3*i + j ].SetValue( str(vv[j]) )


    #################################
    def OnSlice(self,evt):
        """
        """
        print self.CommentHead+"OnSlice"
        Axr = []
        type_axes = []
        foldings = []
        for i in range(4):
            ax = self.SliceAxis[i].GetSelection()
            if ax == 0:
                type_axes.append("X")
            elif ax == 1:
                type_axes.append("Y")
            elif ax == 2:
                type_axes.append("T")
            else:
                raise
            
            if self.SliceFoldCB[i].GetValue():
                foldings.append(1)
            else:
                foldings.append(0)
            
            ar = []
            ar.append( float( self.SliceRange[i*3].GetValue() ) )
            ar.append( float( self.SliceRange[i*3+1].GetValue() ) )
            Axr.append(ar)

        self.D4Mat.sliceMat(Axr[0],Axr[1],Axr[2],Axr[3], type_axes, foldings )

        d2 = M2P.D2Matrix()
        map = d2.ReadMatrix( self.D4Mat.SlicedECM )

        titles = ["","","","",""]
        titles[0] = "D4Matrix Slice"
        
        for i,c in enumerate(type_axes):
            title = self.D4MatTitle[i].GetValue()
            if c=="X":
                titles[2] = title
            elif c=="Y":
                titles[3] = title
            else:
                titles[1] += "Ax%d : %s\n" % (i,title)
                titles[1] += "     from %s to %s\n" % (self.SliceRange[i*3].GetValue(),self.SliceRange[i*3+1].GetValue())
        
        titles[4] = "Intensity"
        
        if self.M2Plot == None:
            self.M2Plot = Plot2DMap(0, "D4MatSlicer", self.frame)
            
        self.M2Plot.PlotMap( titles, map)
        
    #################################
    def OnOut3DText(self,evt):
        """
        """
        print self.CommentHead+"OnSlice"
        # open dialog
        dlg = wx.FileDialog(self.frame, 'Choose 3D text output file ...', os.getcwd(),
                            style=wx.SAVE, wildcard='TEXT(*.txt)|*.txt')
        # cancel?
        if dlg.ShowModal() != wx.ID_OK:
            return
        
        filepath = str( dlg.GetPath() )
        self.StatusTextCtrl.SetLabel( "Data Slicing and save file %s" % (filepath ) )
        
        Axr = []
        type_axes = []
        foldings = []
        for i in range(4):
            ax = self.SliceAxis[i].GetSelection()
            if ax == 0:
                type_axes.append("X")
            elif ax == 1:
                type_axes.append("Y")
            elif ax == 2:
                type_axes.append("T")
            elif ax == 3:
                type_axes.append("Z")
            else:
                raise
            
            if self.SliceFoldCB[i].GetValue():
                foldings.append(1)
            else:
                foldings.append(0)
            
            ar = []
            ar.append( float( self.SliceRange[i*3].GetValue() ) )
            ar.append( float( self.SliceRange[i*3+1].GetValue() ) )
            Axr.append(ar)
            
        import Manyo as mm
        a1v = mm.ListToDoubleVector( Axr[0] )
        a2v = mm.ListToDoubleVector( Axr[1] )
        a3v = mm.ListToDoubleVector( Axr[2] )
        a4v = mm.ListToDoubleVector( Axr[3] )
        def_axes = mm.MakeStringVector()
        fld = mm.ListToInt4Vector( foldings )
        for Axtype in type_axes:
            def_axes.append(Axtype)

        self.D4Mat.D4Mat.OutputText3D(a1v, a2v, a3v, a4v, def_axes, fld, str(filepath) )
        self.StatusTextCtrl.SetLabel( "Ready" )
        
    #################################
    def OnOpenMakeNewFrame(self,evt):
        """
        """
        print self.CommentHead+"OnOpenMakeNewFrame"
        self.StatusTextCtrl.SetLabel( "Opening window for Making New D4Matrix." )
        dlg = MakeNewMatrixFrame(self)
        #self.StatusTextCtrl.SetLabel( "Done." )

        
    #################################
    def OnOpenRebinFrame(self,evt):
        """
        """
        print self.CommentHead+"OnOpenRebinFrame"
        self.StatusTextCtrl.SetLabel( "Opening window for Rebin D4Matrix." )
        dlg = RebinMatrixFrame(self)
        #self.StatusTextCtrl.SetLabel( "Done." )
        

    #################################
    def OnSaveDataAsText(self,evt):
        """
        """
        print self.CommentHead+"OnSaveDataAsText"


    #################################
    def OnClose(self,evt):
        """
        """
        print self.CommentHead+"OnClose"
        self._CloseD4Matrix()
        self.frame.Destroy()


    ################################
    def _CloseD4Matrix(self):
        """
        """
        self.D4Mat.close()
        
        self.ParamFilePath = ""
        self.D4MatrixFilePathCtrl.SetLabel("Not selected.")
        for i in range(2,len(self.ButtonCtrlList)):
            self.ButtonCtrlList[i].Enable(False)
        self.FileListBox.Clear()



#######################################
#  Plot2DMap
####################################### 
class Plot2DMap(object):
    """
    2次元マップ表示クラス
    """ 
    #########################################################
    def __init__(self, num, code, frame):
        """
        コンストラクタ
        @param  num  タブ No.
        @param  code  マップ画面タイトルに付する記号
        @param  frame  親フレーム
        @retval 無し
        """ 
        self.frame = frame
        self.code = code
        self.d2 = None
        if num == 0:
            self.ifi = UtilPlot.IFEvtProp(0)
            self.pno = 0  
        else:
            self.pno = (num+1) * 10  
            self.ifi = UtilPlot.IFEvtProp(self.pno) 

    #########################################
    def PlotMap(self, labels, map):
        """
        2D マップ表示
        @param panel タブのインスタンス
        @param map　2D データ
        @retval 無し
        """
        # D2 プロッタが表示されているか
        if self.d2 == None:
            # 2Dプロッタフレームクローズイベントのリスナー登録
            self.ifi.AddListner('2dclosed',  self.OnNotify2DClose)
            self.d2 = D2Vis.MapD2Frame(self.frame , map, self.pno, self.code) 
        else:
            # データ変更
            self.ifi.NotifyEvent(self, "changedata", map) 

        self.ifi.NotifyEvent(self, "title", (labels[0], labels[1]))
        self.ifi.NotifyEvent(self, "scalelabel", (labels[2], labels[3], labels[4]))

        self.plotNow = (map, labels[0], labels[1])  
 
                
    #########################################
    def OnNotify2DClose(self, *args):
        """
        2Dプロッタクローズイベント受信処理
        @param evt　　イベント
        @retval 無し
        """
        self.d2 = None

    #########################################
    def Request2DClose(self):
        """
        2Dプロッタクローズ要求処理
        @param  無し
        @retval 無し
        """
         # 2Dプロッタが開いているなら
        if self.d2 == None:
             # 2Dプロッタのクローズ要求を出す　
            self.ifi.NotifyEvent(self, "close")


#######################################
#  Splash 
#######################################
class UtsusemiSplashScreen(wx.SplashScreen):
    def __init__(self,splash_path):

        bmp = wx.Image(splash_path).ConvertToBitmap()
        wx.SplashScreen.__init__(self,bmp,
                                 wx.SPLASH_CENTRE_ON_SCREEN|wx.SPLASH_TIMEOUT,
                                 5000, None, -1)
        self.Bind(wx.EVT_CLOSE, self.OnClose)
        self.fc = wx.FutureCall( 2000, self.ShowMain)

    def OnClose(self, evt):
        evt.Skip()
        self.Hide()
        if self.fc.IsRunning():
            self.fc.Stop()
            self.ShowMain()

    def ShowMain(self):
        pass


###############################
if __name__=='__main__':
    app = wx.App(False)
    splash_path = os.path.join( os.environ["UTSUSEMI_BASE_DIR"],"ana","Reduction","LOGO_utsusemi_s.png" )
    if os.path.exists(splash_path):
        splash = UtsusemiSplashScreen(splash_path)
        splash.Show()
    
    frame = D4MatSlicer(None)
    app.MainLoop()



#!/usr/bin/python
import Manyo as m

API = m.AcquireNeutronSourceImageInformation()

API.SetSavePath("./Image")

API.GetImageInformation("BP")
API.GetImageInformation("CT01")
API.GetImageInformation("H2")
API.GetImageInformation("JP")
API.GetImageInformation("ST")
API.GetImageInformation("CT3H")
API.GetImageInformation("CT1D")

#API.GetImageInformation("CT01","2009/06/02 23:00:00","2009/06/03 01:00:00")
#API.GetImageInformation("CT01","2009/09/06 23:00:00")

#API.GetImageInformation("H2","2009/05/31 23:00:00","2009/06/01 01:00:00")
#API.GetImageInformation("H2","2009/09/06 23:00:00")
